/****************************************************************************
 *   Copyright (C) 2009-2010 by Claas Anders "CaScAdE" Rathje               *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include "main.h"
#include "max7456_software_spi.h"
#include "osd_helpers.h"
#include "osd_ncmode_default.h"

#if !(ALLCHARSDEBUG|(WRITECHARS != -1))

int osd_ncmode_minimal() {

    if (COSD_FLAGS_MODES & COSD_FLAG_HUD) {
        // write icons at init or after menu/mode-switch
        if (!(COSD_FLAGS_RUNTIME & COSD_ICONS_WRITTEN)) {
            write_char_xy(4, top_line, 0xCB); // km/h
            write_char_xy(15, top_line, 0xCC); // small meters m height
            write_char_xy(26, top_line, 0xCC); // small meters m height
            write_char_xy(16, top_line, 0xB0); // left circle
            write_char_xy(18, top_line, 0xB2); // right circle
            write_char_xy(7, bottom_line, 0x9E); // small V
            write_char_xy(17, bottom_line, 0xD2); // fly clock
            write_char_xy(26, bottom_line, 0xC8); // sat1
            write_char_xy(27, bottom_line, 0xC9); // sat2
            COSD_FLAGS_RUNTIME |= COSD_ICONS_WRITTEN;
        }

        // first line
        write_ndigit_number_u(1, top_line, (uint16_t)(((uint32_t)naviData.GroundSpeed * (uint32_t)9) / (uint32_t)250), 3, 0);

        write_ndigit_number_u(12, top_line, naviData.HomePositionDeviation.Distance / 10, 3, 0);

        uint16_t heading_home = (naviData.HomePositionDeviation.Bearing + 360 - naviData.CompassHeading) % 360;
        write_char_xy(17, top_line, 0xa0 + heading_fine_conv(heading_home));

        if (naviData.Altimeter > 300 || naviData.Altimeter < -300) {
            // above 10m only write full meters
            write_ndigit_number_s(22, top_line, naviData.Altimeter / 30, 4, 0);
        } else {
            // up to 10m write meters.dm
            write_ndigit_number_s_10th(22, top_line, naviData.Altimeter / 3, 3, 0);
        }

        draw_variometer(27, top_line, naviData.Variometer);


        // center
        if (naviData.FCFlags & FLAG_MOTOR_RUN) { // should be engines running
            if (!(old_MKFlags & FLAG_MOTOR_RUN)) { // motors just started, clear middle
                clear();
                // update flags to paint display again if needed
                COSD_FLAGS_RUNTIME &= ~COSD_ICONS_WRITTEN;
            }
            if (COSD_FLAGS_MODES & COSD_FLAG_ARTHORIZON) {
                if (COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) {
                    draw_artificial_horizon(top_line + 2, bottom_line - 2, naviData.AngleNick, naviData.AngleRoll);
                } else {
                    draw_artificial_horizon(top_line + 2, bottom_line - 1, naviData.AngleNick, naviData.AngleRoll);
                }
            }
            // motors are on, assume we were/are flying
            COSD_FLAGS_RUNTIME |= COSD_WASFLYING;
        } else {
            // stats
            if ((COSD_FLAGS_RUNTIME & COSD_WASFLYING) && (COSD_FLAGS_MODES & COSD_FLAG_STATS)) {
                uint8_t line = 3;
                write_ascii_string_pgm(2, line, (const char *)(pgm_read_word(&(stats_item_pointers[0])))); // max Altitude
                write_ndigit_number_s(18, line, max_Altimeter / 30, 4, 0);
                write_char_xy(22, line, 204); // small meters m
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[1])))); // max Speed
                write_ndigit_number_u(19, line, (uint16_t)(((uint32_t)max_GroundSpeed * (uint32_t)9) / (uint32_t)250), 3, 0);
                write_char_xy(22, line, 203); // km/h
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[2])))); // max Distance
                write_ndigit_number_u(19, line, max_Distance / 10, 3, 0);
                write_char_xy(22, line, 204); // small meters m
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[3])))); // min voltage
                write_ndigit_number_u_10th(18, line, min_UBat, 3, 0);
                write_char_xy(22, line, 0x9E); // small V
                if (COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) {
                    write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[7])))); // ampere
                    write_ndigit_number_u_10th(18, line, max_ampere / 10, 3, 0);
                    write_char_xy(22, line, 0x9F); // small A
                }
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[4])))); // max time
                write_time(16, line, max_FlyingTime);
                write_char_xy(22, line, 210); // fly clock
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[5])))); // longitude
                write_gps_pos(15, line, naviData.CurrentPosition.Longitude);
                write_ascii_string_pgm(2, ++line, (const char *)(pgm_read_word(&(stats_item_pointers[6])))); // latitude
                write_gps_pos(15, line, naviData.CurrentPosition.Latitude);
            } else if (COSD_FLAGS_MODES & COSD_FLAG_ARTHORIZON) { // if no stats there is space horizon
                if (COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) {
                    draw_artificial_horizon(top_line + 2, bottom_line - 2, naviData.AngleNick, naviData.AngleRoll);
                } else {
                    draw_artificial_horizon(top_line + 2, bottom_line - 1, naviData.AngleNick, naviData.AngleRoll);
                }
            }
        }
        if (COSD_FLAGS_MODES & COSD_FLAG_BIGVARIO) {
            draw_big_variometer(27, 8, naviData.Variometer);
        }

        // pre-bottom line
        if (COSD_FLAGS_RUNTIME & COSD_FLAG_STROMREC) {
            //write_ndigit_number_s(3, bottom_line - 1, ampere, 4, 0);
            write_ndigit_number_u_10th(3, bottom_line - 1, ampere / 10, 3, 0);
            write_ndigit_number_s(10, bottom_line - 1, ampere_wasted / 10, 4, 0);
        }

        // bottom line
        draw_battery(2, bottom_line, min_voltage, naviData.UBat, max_voltage);
        write_ndigit_number_u_10th(3, bottom_line, naviData.UBat, 3, 0);
        if (naviData.UBat <= min_voltage && last_UBat > min_voltage) {
            for (uint8_t x = 2; x < 8; x++)
                write_char_att_xy(x, bottom_line, BLINK);
        } else if (naviData.UBat > min_voltage && last_UBat < min_voltage) {
            for (uint8_t x = 2; x < 8; x++)
                write_char_att_xy(x, bottom_line, 0);
        }

        write_time(11, bottom_line, naviData.FlyingTime);

        write_ndigit_number_u(24, bottom_line, naviData.SatsInUse, 2, 0);

        if (naviData.NCFlags & NC_FLAG_CH) {
            write_char_xy(27, bottom_line, 231); // gps ch
        } else if (naviData.NCFlags & NC_FLAG_PH) {
            write_char_xy(27, bottom_line, 230); // gps ph
        } else { // (naviData.NCFlags & NC_FLAG_FREE)
            write_char_xy(27, bottom_line, 201); // sat2 (free)
        }
    }

    // remember statistics (only when engines running)
    if (naviData.FCFlags & FLAG_MOTOR_RUN) {
        if (naviData.Altimeter > max_Altimeter) max_Altimeter = naviData.Altimeter;
        if (naviData.GroundSpeed > max_GroundSpeed) max_GroundSpeed = naviData.GroundSpeed;
        if (naviData.HomePositionDeviation.Distance > max_Distance) {
            max_Distance = naviData.HomePositionDeviation.Distance;
        }
        if (naviData.UBat < min_UBat) min_UBat = naviData.UBat;
        if (naviData.FlyingTime > max_FlyingTime) max_FlyingTime = naviData.FlyingTime;
        if (ampere > max_ampere) max_ampere = ampere;
    }

    // remember last values
    last_UBat = naviData.UBat;
    old_MKFlags = naviData.FCFlags;
    seconds_since_last_data = 0;

    return 0;
}

#endif
