/****************************************************************************
 *   Copyright (C) 2009-2010 by Claas Anders "CaScAdE" Rathje               *
 *   admiralcascade@gmail.com                                               *
 *   Project-URL: http://www.mylifesucks.de/oss/c-osd/                      *
 *                                                                          *
 *   This program is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation; either version 2 of the License.         *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to the                          *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "main.h"
#include "max7456_software_spi.h"


#if (ALLCHARSDEBUG|(WRITECHARS != -1))

/**
 * easy char creation:
 * http://www.mylifesucks.de/tools/max7456/
 */
// chars 0 - 255 with 54 byte per char
// orginal filename: c-osd.mcm 
// Mon, 12 Apr 2010 19:12:55 +0200



//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr0[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	-----##-----
//	 3	----#  #----
//	 4	---#   #----
//	 5	--#    #----
//	 6	--#    #----
//	 7	--###  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	---##  ##---
//	15	--#      #--
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr1[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x50, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0x28, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#    ###   #
//	 6	#   #---#  #
//	 7	-###----#  #
//	 8	-------#   #
//	 9	------#   #-
//	10	-----#   #--
//	11	----#   #---
//	12	---#   #----
//	13	--#   #-----
//	14	-#   ######-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr2[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x80, 0xa8, 0x2a, 0x15, 0x28, 0x40, 0x55, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x80, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	-##-----#  #
//	 8	-----###   #
//	 9	----#     #-
//	10	----#     #-
//	11	-----###   #
//	12	-##-----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr3[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x28, 0x55, 0x40, 0xa8, 0x55, 0x2a, 0xa1, 0x55, 0x2a, 0xa1, 0x55, 0x40, 0xa8, 0x41, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-------##---
//	 3	------#  #--
//	 4	-----#   #--
//	 5	----#    #--
//	 6	---#     #--
//	 7	--#   #  #--
//	 8	-#   ##  #--
//	 9	#   #-#  #--
//	10	#  ####  ##-
//	11	#          #
//	12	#          #
//	13	-######  ##-
//	14	------#  #--
//	15	------#  #--
//	16	------#  #--
//	17	-------##---
//		000000000011
//		012345678901
const char chr4[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x15, 0x55, 0x52, 0x85, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x85, 0x54, 0xaa, 0x85, 0x52, 0xa2, 0x85, 0x4a, 0x82, 0x85, 0x2a, 0x12, 0x85, 0x28, 0x02, 0x81, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x02, 0x81, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	#  ########-
//	 6	#  ######---
//	 7	#        #--
//	 8	#         #-
//	 9	-#######   #
//	10	--------#  #
//	11	--------#  #
//	12	-##-----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr5[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0x00, 0x01, 0x28, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x40, 0x00, 0xa8, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x41, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #-----##-
//	 8	#  ######---
//	 9	#        #--
//	10	#         #-
//	11	#  #####   #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr6[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x41, 0x28, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	-########  #
//	 6	-------#   #
//	 7	------#   #-
//	 8	-----#   #--
//	 9	----#   #---
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr7[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#   ####   #
//	 9	-#        #-
//	10	-#        #-
//	11	#   ####   #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr8[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#   #####  #
//	 9	-#         #
//	10	--#        #
//	11	---######  #
//	12	-##-----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr9[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0x28, 0x4a, 0xaa, 0xa8, 0x52, 0xaa, 0xa8, 0x54, 0x00, 0x28, 0x41, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr10[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-----##-----
//	 3	----#  #----
//	 4	---#    #---
//	 5	--#      #--
//	 6	-#   ##   #-
//	 7	#   #--#   #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  ######  #
//	11	#          #
//	12	#          #
//	13	#  ######  #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr11[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0x82, 0xa1, 0x2a, 0x14, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0x28, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0x00, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	-########---
//	 3	#        #--
//	 4	#         #-
//	 5	#  #####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #####   #
//	 9	#         #-
//	10	#         #-
//	11	#  #####   #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#  #####   #
//	15	#         #-
//	16	#        #--
//	17	-########---
//		000000000011
//		012345678901
const char chr12[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0xa8, 0x2a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0xa8, 0x2a, 0xaa, 0xa1, 0x2a, 0xaa, 0x85, 0x40, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #-----##-
//	 8	#  #--------
//	 9	#  #--------
//	10	#  #--------
//	11	#  #--------
//	12	#  #-----##-
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr13[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x41, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x41, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-######-----
//	 3	#      #----
//	 4	#       #---
//	 5	#  ###   #--
//	 6	#  #--#   #-
//	 7	#  #---#   #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #---#   #
//	13	#  #--#   #-
//	14	#  ###   #--
//	15	#       #---
//	16	#      #----
//	17	-######-----
//		000000000011
//		012345678901
const char chr14[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x01, 0x55, 0x2a, 0xa8, 0x55, 0x2a, 0xaa, 0x15, 0x28, 0x0a, 0x85, 0x28, 0x52, 0xa1, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x54, 0xa8, 0x28, 0x52, 0xa1, 0x28, 0x0a, 0x85, 0x2a, 0xaa, 0x15, 0x2a, 0xa8, 0x55, 0x40, 0x01, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	#  ########-
//	 6	#  #--------
//	 7	#  #--------
//	 8	#  ######---
//	 9	#        #--
//	10	#        #--
//	11	#  ######---
//	12	#  #--------
//	13	#  #--------
//	14	#  ########-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr15[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0x00, 0x01, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0x85, 0x28, 0x00, 0x15, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	#  ########-
//	 6	#  #--------
//	 7	#  #--------
//	 8	#  ######---
//	 9	#        #--
//	10	#        #--
//	11	#  ######---
//	12	#  #--------
//	13	#  #--------
//	14	#  #--------
//	15	#  #--------
//	16	#  #--------
//	17	-##---------
//		000000000011
//		012345678901
const char chr16[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0x00, 0x01, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0x85, 0x28, 0x00, 0x15, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ###   #-
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #-----##-
//	 9	#  #-######-
//	10	#  ##      #
//	11	#  ##      #
//	12	#  #-####  #
//	13	#  #----#  #
//	14	#   #####  #
//	15	-#         #
//	16	--#        #
//	17	---########-
//		000000000011
//		012345678901
const char chr17[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x02, 0xa1, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x41, 0x28, 0x40, 0x01, 0x28, 0x2a, 0xa8, 0x28, 0x2a, 0xa8, 0x28, 0x40, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0x28, 0x4a, 0xaa, 0xa8, 0x52, 0xaa, 0xa8, 0x54, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  ######  #
//	 9	#          #
//	10	#          #
//	11	#  ######  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr18[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0x28, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0x00, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	--#      #--
//	 5	---##  ##---
//	 6	----#  #----
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	---##  ##---
//	15	--#      #--
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr19[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x54, 0x28, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0x28, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-----######-
//	 3	----#      #
//	 4	----#      #
//	 5	-----##  ##-
//	 6	------#  #--
//	 7	------#  #--
//	 8	------#  #--
//	 9	------#  #--
//	10	------#  #--
//	11	------#  #--
//	12	-##---#  #--
//	13	#  #--#  #--
//	14	#   ##   #--
//	15	-#      #---
//	16	--#    #----
//	17	---####-----
//		000000000011
//		012345678901
const char chr20[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x01, 0x55, 0x2a, 0xa8, 0x55, 0x2a, 0xa8, 0x55, 0x42, 0x81, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x41, 0x52, 0x85, 0x28, 0x52, 0x85, 0x2a, 0x0a, 0x85, 0x4a, 0xaa, 0x15, 0x52, 0xa8, 0x55, 0x54, 0x01, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #---#   #
//	 5	#  #--#   #-
//	 6	#  #-#   #--
//	 7	#  ##   #---
//	 8	#  #   #----
//	 9	#     #-----
//	10	#     #-----
//	11	#  #   #----
//	12	#  ##   #---
//	13	#  #-#   #--
//	14	#  #--#   #-
//	15	#  #---#   #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr21[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x54, 0xa8, 0x28, 0x52, 0xa1, 0x28, 0x4a, 0x85, 0x28, 0x2a, 0x15, 0x28, 0xa8, 0x55, 0x2a, 0xa1, 0x55, 0x2a, 0xa1, 0x55, 0x28, 0xa8, 0x55, 0x28, 0x2a, 0x15, 0x28, 0x4a, 0x85, 0x28, 0x52, 0xa1, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	-##---------
//	 3	#  #--------
//	 4	#  #--------
//	 5	#  #--------
//	 6	#  #--------
//	 7	#  #--------
//	 8	#  #--------
//	 9	#  #--------
//	10	#  #--------
//	11	#  #--------
//	12	#  #--------
//	13	#  #--------
//	14	#  ########-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr22[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#   #--#   #
//	 5	#    ##    #
//	 6	#          #
//	 7	#  #    #  #
//	 8	#  ##  ##  #
//	 9	#  ##  ##  #
//	10	#  #-##-#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr23[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x2a, 0x82, 0xa8, 0x2a, 0xaa, 0xa8, 0x28, 0xaa, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x41, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#   #---#  #
//	 5	#    #--#  #
//	 6	#    #--#  #
//	 7	#  #  #-#  #
//	 8	#  #  #-#  #
//	 9	#  ##  ##  #
//	10	#  ##  ##  #
//	11	#  #-#  #  #
//	12	#  #-#  #  #
//	13	#  #--#    #
//	14	#  #--#    #
//	15	#  #---#   #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr24[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x2a, 0x15, 0x28, 0x2a, 0x85, 0x28, 0x2a, 0x85, 0x28, 0x28, 0xa1, 0x28, 0x28, 0xa1, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x4a, 0x28, 0x28, 0x4a, 0x28, 0x28, 0x52, 0xa8, 0x28, 0x52, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr25[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-########---
//	 3	#        #--
//	 4	#         #-
//	 5	#  #####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #####   #
//	 9	#         #-
//	10	#        #--
//	11	#  ######---
//	12	#  #--------
//	13	#  #--------
//	14	#  #--------
//	15	#  #--------
//	16	#  #--------
//	17	-##---------
//		000000000011
//		012345678901
const char chr26[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0xa8, 0x2a, 0xaa, 0xa1, 0x2a, 0xaa, 0x85, 0x28, 0x00, 0x15, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #-##-#  #
//	11	#  ##  ##  #
//	12	#  ##   #  #
//	13	#  #-#     #
//	14	#   ###   #-
//	15	-#         #
//	16	--#    ##  #
//	17	---####--##-
//		000000000011
//		012345678901
const char chr27[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x41, 0x28, 0x28, 0x28, 0x28, 0x28, 0x2a, 0x28, 0x28, 0x4a, 0xa8, 0x2a, 0x02, 0xa1, 0x4a, 0xaa, 0xa8, 0x52, 0xa8, 0x28, 0x54, 0x01, 0x41};


//	 0	------------
//	 1	------------
//	 2	-########---
//	 3	#        #--
//	 4	#         #-
//	 5	#  #####   #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #####   #
//	 9	#         #-
//	10	#        #--
//	11	#  #   ##---
//	12	#  ##   #---
//	13	#  #-#   #--
//	14	#  #--#   #-
//	15	#  #---#   #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr28[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x15, 0x2a, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x00, 0xa8, 0x2a, 0xaa, 0xa1, 0x2a, 0xaa, 0x85, 0x28, 0xa8, 0x15, 0x28, 0x2a, 0x15, 0x28, 0x4a, 0x85, 0x28, 0x52, 0xa1, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	---#######--
//	 3	--#       #-
//	 4	-#         #
//	 5	#   #####  #
//	 6	#  #-----##-
//	 7	#  #--------
//	 8	#   #####---
//	 9	-#       #--
//	10	--#       #-
//	11	---#####   #
//	12	--------#  #
//	13	-##-----#  #
//	14	#  #####   #
//	15	#         #-
//	16	-#       #--
//	17	--#######---
//		000000000011
//		012345678901
const char chr29[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x05, 0x52, 0xaa, 0xa1, 0x4a, 0xaa, 0xa8, 0x2a, 0x00, 0x28, 0x28, 0x55, 0x41, 0x28, 0x55, 0x55, 0x2a, 0x00, 0x15, 0x4a, 0xaa, 0x85, 0x52, 0xaa, 0xa1, 0x54, 0x00, 0xa8, 0x55, 0x55, 0x28, 0x41, 0x55, 0x28, 0x28, 0x00, 0xa8, 0x2a, 0xaa, 0xa1, 0x4a, 0xaa, 0x85, 0x50, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	-####  ####-
//	 6	----#  #----
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr30[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x28, 0x01, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr31[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#   #--#   #
//	13	-#   ##   #-
//	14	--#      #--
//	15	---#    #---
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr32[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #-##-#  #
//	 9	#  ##  ##  #
//	10	#  ##  ##  #
//	11	#  ##  ##  #
//	12	#  ##  ##  #
//	13	#  ##  ##  #
//	14	#          #
//	15	-#        #-
//	16	--#  ##  #--
//	17	---##--##---
//		000000000011
//		012345678901
const char chr33[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x41, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0x82, 0x85, 0x54, 0x14, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#   #--#   #
//	 7	-#   ##   #-
//	 8	--#      #--
//	 9	---#    #---
//	10	---#    #---
//	11	--#      #--
//	12	-#   ##   #-
//	13	#   #--#   #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr34[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0x82, 0xa1, 0x2a, 0x14, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	-##------##-
//	 3	#  #----#  #
//	 4	#  #----#  #
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#   #--#   #
//	 9	-#   ##   #-
//	10	--#      #--
//	11	---#    #---
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr35[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##########-
//	 3	#          #
//	 4	#          #
//	 5	-########  #
//	 6	-------#   #
//	 7	------#   #-
//	 8	-----#   #--
//	 9	----#   #---
//	10	---#   #----
//	11	--#   #-----
//	12	-#   #------
//	13	#   #-------
//	14	#  ########-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr36[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x28, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---######---
//	 5	--#      #--
//	 6	--#       #-
//	 7	---#####   #
//	 8	---######  #
//	 9	--#     #  #
//	10	-#         #
//	11	#   ####   #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#         #
//	16	--#     #  #
//	17	---#####-##-
//		000000000011
//		012345678901
const char chr37[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0xa1, 0x54, 0x00, 0xa8, 0x54, 0x00, 0x28, 0x52, 0xaa, 0x28, 0x4a, 0xaa, 0xa8, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa8, 0x52, 0xaa, 0x28, 0x54, 0x00, 0x41};


//	 0	------------
//	 1	------------
//	 2	-##---------
//	 3	#  #--------
//	 4	#  #--------
//	 5	#  #-###----
//	 6	#  ##   #---
//	 7	#  #     #--
//	 8	#     #   #-
//	 9	#    #-#   #
//	10	#   #---#  #
//	11	#   #---#  #
//	12	#   #---#  #
//	13	#    #-#   #
//	14	#     #   #-
//	15	#  #     #--
//	16	#  ##   #---
//	17	-##--###----
//		000000000011
//		012345678901
const char chr38[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x40, 0x55, 0x28, 0x2a, 0x15, 0x28, 0xaa, 0x85, 0x2a, 0xa2, 0xa1, 0x2a, 0x84, 0xa8, 0x2a, 0x15, 0x28, 0x2a, 0x15, 0x28, 0x2a, 0x15, 0x28, 0x2a, 0x84, 0xa8, 0x2a, 0xa2, 0xa1, 0x28, 0xaa, 0x85, 0x28, 0x2a, 0x15, 0x41, 0x40, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---######---
//	 6	--#      #--
//	 7	-#        #-
//	 8	#   ####   #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #-----##-
//	12	#  #-----##-
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr39[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x41, 0x28, 0x55, 0x41, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	---------##-
//	 3	--------#  #
//	 4	--------#  #
//	 5	----###-#  #
//	 6	---#   ##  #
//	 7	--#     #  #
//	 8	-#   #     #
//	 9	#   #-#    #
//	10	#  #---#   #
//	11	#  #---#   #
//	12	#  #---#   #
//	13	#   #-#    #
//	14	-#   #     #
//	15	--#     #  #
//	16	---#   ##  #
//	17	----###--##-
//		000000000011
//		012345678901
const char chr40[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x01, 0x28, 0x54, 0xa8, 0x28, 0x52, 0xaa, 0x28, 0x4a, 0x8a, 0xa8, 0x2a, 0x12, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x54, 0xa8, 0x2a, 0x12, 0xa8, 0x4a, 0x8a, 0xa8, 0x52, 0xaa, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x01, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---######---
//	 6	--#      #--
//	 7	-#        #-
//	 8	#   ####   #
//	 9	#  ######  #
//	10	#          #
//	11	#         #-
//	12	#  #######--
//	13	#  #----###-
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr41[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x00, 0x28, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa1, 0x28, 0x00, 0x05, 0x28, 0x55, 0x01, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------####--
//	 3	-----#    #-
//	 4	----#     #-
//	 5	----#  ###--
//	 6	--###  ###--
//	 7	-#        #-
//	 8	-#        #-
//	 9	--###  ###--
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr42[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x05, 0x55, 0x4a, 0xa1, 0x55, 0x2a, 0xa1, 0x55, 0x28, 0x05, 0x50, 0x28, 0x05, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x50, 0x28, 0x05, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---######---
//	 5	--#      #--
//	 6	-#        #-
//	 7	#   ####   #
//	 8	#  #----#  #
//	 9	#   ####   #
//	10	-#         #
//	11	--#     #  #
//	12	---######  #
//	13	--##----#  #
//	14	-#  ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr43[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa8, 0x52, 0xaa, 0x28, 0x54, 0x00, 0x28, 0x50, 0x55, 0x28, 0x4a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-##---------
//	 3	#  #--------
//	 4	#  #--------
//	 5	#  #-###----
//	 6	#  ##   #---
//	 7	#  #     #--
//	 8	#     #   #-
//	 9	#    #-#   #
//	10	#   #---#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr44[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x40, 0x55, 0x28, 0x2a, 0x15, 0x28, 0xaa, 0x85, 0x2a, 0xa2, 0xa1, 0x2a, 0x84, 0xa8, 0x2a, 0x15, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----##-----
//	 4	----#  #----
//	 5	----#  #----
//	 6	-----##-----
//	 7	---####-----
//	 8	--#    #----
//	 9	--#    #----
//	10	---##  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#   #---
//	15	-----#   #--
//	16	------#  #--
//	17	-------##---
//		000000000011
//		012345678901
const char chr45[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x54, 0x01, 0x55, 0x52, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x54, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x2a, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15};


//	 0	------------
//	 1	-------##---
//	 2	------#  #--
//	 3	------#  #--
//	 4	-------##---
//	 5	-----####---
//	 6	----#    #--
//	 7	----#    #--
//	 8	-----##  #--
//	 9	------#  #--
//	10	------#  #--
//	11	------#  #--
//	12	------#  #--
//	13	------#  #--
//	14	---###   #--
//	15	--#     #---
//	16	--#    #----
//	17	---####-----
//		000000000011
//		012345678901
const char chr46[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x54, 0x15, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15, 0x55, 0x40, 0x15, 0x55, 0x2a, 0x85, 0x55, 0x2a, 0x85, 0x55, 0x42, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x54, 0x0a, 0x85, 0x52, 0xaa, 0x15, 0x52, 0xa8, 0x55, 0x54, 0x01, 0x55};


//	 0	------------
//	 1	------------
//	 2	-##---------
//	 3	#  #--------
//	 4	#  #--------
//	 5	#  #-----##-
//	 6	#  #----#  #
//	 7	#  #---#   #
//	 8	#  #--#   #-
//	 9	#  #-#   #--
//	10	#  ##   #---
//	11	#  #   #----
//	12	#       #---
//	13	#    #   #--
//	14	#   #-#   #-
//	15	#  #---#   #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr47[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x54, 0xa8, 0x28, 0x52, 0xa1, 0x28, 0x4a, 0x85, 0x28, 0x2a, 0x15, 0x28, 0xa8, 0x55, 0x2a, 0xaa, 0x15, 0x2a, 0x8a, 0x85, 0x2a, 0x12, 0xa1, 0x28, 0x54, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	-----##-----
//	 3	----#  #----
//	 4	----#  #----
//	 5	----#  #----
//	 6	----#  #----
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr48[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##-#####---
//	 6	#  #     #--
//	 7	#         #-
//	 8	#   #  #   #
//	 9	#  ##  ##  #
//	10	#  ##  ##  #
//	11	#  ##  ##  #
//	12	#  ##  ##  #
//	13	#  ##  ##  #
//	14	#  ##  ##  #
//	15	#  ##  ##  #
//	16	#  ##  ##  #
//	17	-##--##--##-
//		000000000011
//		012345678901
const char chr49[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x00, 0x15, 0x28, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x2a, 0x28, 0xa8, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x41, 0x41, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##-#####---
//	 6	#  #     #--
//	 7	#         #-
//	 8	#   ####   #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#  #----#  #
//	15	#  #----#  #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr50[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x00, 0x15, 0x28, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---######---
//	 6	--#      #--
//	 7	-#        #-
//	 8	#   ####   #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr51[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-##-#####---
//	 5	#  #     #--
//	 6	#         #-
//	 7	#    ###   #
//	 8	#   #---#  #
//	 9	#   #---#  #
//	10	#   #---#  #
//	11	#    ###   #
//	12	#         #-
//	13	#  #     #--
//	14	#  ######---
//	15	#  #--------
//	16	#  #--------
//	17	-##---------
//		000000000011
//		012345678901
const char chr52[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x00, 0x15, 0x28, 0xaa, 0x85, 0x2a, 0xaa, 0xa1, 0x2a, 0x80, 0xa8, 0x2a, 0x15, 0x28, 0x2a, 0x15, 0x28, 0x2a, 0x15, 0x28, 0x2a, 0x80, 0xa8, 0x2a, 0xaa, 0xa1, 0x28, 0xaa, 0x85, 0x28, 0x00, 0x15, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---#####-##-
//	 5	--#     #  #
//	 6	-#         #
//	 7	#   ###    #
//	 8	#  #---#   #
//	 9	#  #---#   #
//	10	#  #---#   #
//	11	#   ###    #
//	12	-#         #
//	13	--#    #   #
//	14	---####-#  #
//	15	--------#  #
//	16	--------#  #
//	17	---------##-
//		000000000011
//		012345678901
const char chr53[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x41, 0x52, 0xaa, 0x28, 0x4a, 0xaa, 0xa8, 0x2a, 0x02, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x54, 0xa8, 0x2a, 0x02, 0xa8, 0x4a, 0xaa, 0xa8, 0x52, 0xa8, 0xa8, 0x54, 0x01, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##--####---
//	 6	#  ##    #--
//	 7	#  #      #-
//	 8	#     ##   #
//	 9	#    #--#  #
//	10	#   #----##-
//	11	#  #--------
//	12	#  #--------
//	13	#  #--------
//	14	#  #--------
//	15	#  #--------
//	16	#  #--------
//	17	-##---------
//		000000000011
//		012345678901
const char chr54[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x40, 0x15, 0x28, 0x2a, 0x85, 0x28, 0xaa, 0xa1, 0x2a, 0xa0, 0xa8, 0x2a, 0x85, 0x28, 0x2a, 0x15, 0x41, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---######---
//	 6	--#      #--
//	 7	-#        #-
//	 8	#   ####   #
//	 9	#   ###-#  #
//	10	-#     ####-
//	11	--##     #--
//	12	-##-##    #-
//	13	#  #--##   #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr55[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x2a, 0x01, 0x28, 0x4a, 0xa8, 0x01, 0x50, 0xaa, 0x85, 0x41, 0x0a, 0xa1, 0x28, 0x50, 0xa8, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	-----##-----
//	 3	----#  #----
//	 4	----#  #----
//	 5	---##  ##---
//	 6	--#      #--
//	 7	--#      #--
//	 8	---##  ##---
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#   #---
//	15	-----#   #--
//	16	------#  #--
//	17	-------##---
//		000000000011
//		012345678901
const char chr56[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0x28, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x54, 0x28, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x2a, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##------##-
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#  #----#  #
//	13	#  #----#  #
//	14	#   ####   #
//	15	-#        #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr57[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x00, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##------##-
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #----#  #
//	10	#  #----#  #
//	11	#  #----#  #
//	12	#   #--#   #
//	13	-#   ##   #-
//	14	--#      #--
//	15	---#    #---
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr58[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##------##-
//	 6	#  #----#  #
//	 7	#  #----#  #
//	 8	#  #----#  #
//	 9	#  #-##-#  #
//	10	#  ##  ##  #
//	11	#  ##  ##  #
//	12	#  ##  ##  #
//	13	#  ##  ##  #
//	14	#   #  #   #
//	15	-#        #-
//	16	--#  ##  #--
//	17	---##--##---
//		000000000011
//		012345678901
const char chr59[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x41, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x2a, 0x28, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0x82, 0x85, 0x54, 0x14, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##------##-
//	 6	#  #----#  #
//	 7	#   #--#   #
//	 8	-#   ##   #-
//	 9	--#      #--
//	10	---#    #---
//	11	----#  #----
//	12	---#    #---
//	13	--#      #--
//	14	-#   ##   #-
//	15	#   #--#   #
//	16	#  #----#  #
//	17	-##------##-
//		000000000011
//		012345678901
const char chr60[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0x82, 0xa1, 0x2a, 0x14, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x41};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-##------##-
//	 5	#  #----#  #
//	 6	#  #----#  #
//	 7	#  #---#   #
//	 8	#  #---#   #
//	 9	#  #--#    #
//	10	#   ##     #
//	11	-#      #  #
//	12	--#    ##  #
//	13	---####-#  #
//	14	---#####   #
//	15	--#       #-
//	16	--#      #--
//	17	---######---
//		000000000011
//		012345678901
const char chr61[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x41, 0x28, 0x55, 0x28, 0x28, 0x55, 0x28, 0x28, 0x54, 0xa8, 0x28, 0x54, 0xa8, 0x28, 0x52, 0xa8, 0x2a, 0x0a, 0xa8, 0x4a, 0xaa, 0x28, 0x52, 0xa8, 0x28, 0x54, 0x01, 0x28, 0x54, 0x00, 0xa8, 0x52, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-##########-
//	 6	#          #
//	 7	#          #
//	 8	-#######   #
//	 9	------#   #-
//	10	-----#   #--
//	11	----#   #---
//	12	---#   #----
//	13	--#   #-----
//	14	-#   ######-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr62[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x80, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--------##--
//	 4	-------#  #-
//	 5	------#  #--
//	 6	-----#  #---
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	-----#  #---
//	15	------#  #--
//	16	-------#  #-
//	17	--------##--
//		000000000011
//		012345678901
const char chr63[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x05, 0x55, 0x54, 0xa1, 0x55, 0x52, 0x85, 0x55, 0x4a, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x4a, 0x15, 0x55, 0x52, 0x85, 0x55, 0x54, 0xa1, 0x55, 0x55, 0x05};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	---##-------
//	 4	--#  #------
//	 5	---#  #-----
//	 6	----#  #----
//	 7	-----#  #---
//	 8	-----#  #---
//	 9	-----#  #---
//	10	-----#  #---
//	11	-----#  #---
//	12	-----#  #---
//	13	-----#  #---
//	14	----#  #----
//	15	---#  #-----
//	16	--#  #------
//	17	---##-------
//		000000000011
//		012345678901
const char chr64[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x15, 0x55, 0x52, 0x85, 0x55, 0x54, 0xa1, 0x55, 0x55, 0x28, 0x55, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x28, 0x55, 0x54, 0xa1, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	-------##---
//	15	------#  #--
//	16	------#  #--
//	17	-------##---
//		000000000011
//		012345678901
const char chr65[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x15, 0x55, 0x52, 0x85, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15};


//	 0	------------
//	 1	------------
//	 2	---######---
//	 3	--#      #--
//	 4	-#        #-
//	 5	#   ####   #
//	 6	#  #----#  #
//	 7	-##-----#  #
//	 8	-------#   #
//	 9	------#   #-
//	10	-----#   #--
//	11	----#   #---
//	12	----#  #----
//	13	-----##-----
//	14	-----##-----
//	15	----#  #----
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr66[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0x00, 0xa8, 0x28, 0x55, 0x28, 0x41, 0x55, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-----##-----
//	 5	----#  #----
//	 6	----#  #----
//	 7	-----##-----
//	 8	------------
//	 9	------------
//	10	-----##-----
//	11	----#  #----
//	12	----#  #----
//	13	---#  #-----
//	14	--#  #------
//	15	---##-------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr67[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0xa1, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-----##-----
//	 5	----#  #----
//	 6	----#  #----
//	 7	-----##-----
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	-----##-----
//	13	----#  #----
//	14	----#  #----
//	15	-----##-----
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr68[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	-----##-----
//	13	----#  #----
//	14	----#  #----
//	15	---#  #-----
//	16	--#  #------
//	17	---##-------
//		000000000011
//		012345678901
const char chr69[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0xa1, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15, 0x55};


//	 0	------------
//	 1	------#-----
//	 2	-----# #----
//	 3	----#   #---
//	 4	-----#  #---
//	 5	------# #---
//	 6	------# #---
//	 7	-----# #----
//	 8	------#-----
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr70[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x51, 0x55, 0x55, 0x48, 0x55, 0x55, 0x2a, 0x15, 0x55, 0x4a, 0x15, 0x55, 0x52, 0x15, 0x55, 0x52, 0x15, 0x55, 0x48, 0x55, 0x55, 0x51, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---------##-
//	 5	--------#  #
//	 6	-------#   #
//	 7	------#   #-
//	 8	-----#   #--
//	 9	----#   #---
//	10	---#   #----
//	11	--#   #-----
//	12	-#   #------
//	13	#   #-------
//	14	#  #--------
//	15	-##---------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr71[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	---#----#---
//	 2	--# #--# #--
//	 3	-#   ##   #-
//	 4	--#  #-#  #-
//	 5	---# #--# #-
//	 6	---# #--# #-
//	 7	--# #--# #--
//	 8	---#----#---
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr72[] PROGMEM = {0x55, 0x55, 0x55, 0x54, 0x55, 0x15, 0x52, 0x14, 0x85, 0x4a, 0x82, 0xa1, 0x52, 0x84, 0xa1, 0x54, 0x85, 0x21, 0x54, 0x85, 0x21, 0x52, 0x14, 0x85, 0x54, 0x55, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	----######--
//	 8	---#      #-
//	 9	---#      #-
//	10	----######--
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr73[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x05, 0x54, 0xaa, 0xa1, 0x54, 0xaa, 0xa1, 0x55, 0x00, 0x05, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	--------##--
//	 3	-------#  #-
//	 4	------#   #-
//	 5	-----#   #--
//	 6	----#   #---
//	 7	---#   #----
//	 8	--#   #-----
//	 9	-#   #------
//	10	-#   #------
//	11	--#   #-----
//	12	---#   #----
//	13	----#   #---
//	14	-----#   #--
//	15	------#   #-
//	16	-------#  #-
//	17	--------##--
//		000000000011
//		012345678901
const char chr74[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x05, 0x55, 0x54, 0xa1, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x54, 0xa8, 0x55, 0x55, 0x2a, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x54, 0xa1, 0x55, 0x55, 0x05};


//	 0	------------
//	 1	------------
//	 2	--##--------
//	 3	-#  #-------
//	 4	-#   #------
//	 5	--#   #-----
//	 6	---#   #----
//	 7	----#   #---
//	 8	-----#   #--
//	 9	------#   #-
//	10	------#   #-
//	11	-----#   #--
//	12	----#   #---
//	13	---#   #----
//	14	--#   #-----
//	15	-#   #------
//	16	-#  #-------
//	17	--##--------
//		000000000011
//		012345678901
const char chr75[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x55, 0x55, 0x4a, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x54, 0xa8, 0x55, 0x55, 0x2a, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x2a, 0x15, 0x54, 0xa8, 0x55, 0x52, 0xa1, 0x55, 0x4a, 0x85, 0x55, 0x4a, 0x15, 0x55, 0x50, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	----####----
//	 5	---#    #---
//	 6	--# #### #--
//	 7	-# ###-## #-
//	 8	# ##  # ## #
//	 9	# # ##  ## #
//	10	# # ### ## #
//	11	# # ##  ## #
//	12	# ##  # # #-
//	13	# #-##-# #--
//	14	-# #----##--
//	15	--# ##### #-
//	16	---#     #--
//	17	----#####---
//		000000000011
//		012345678901
const char chr76[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x55, 0x54, 0xaa, 0x15, 0x52, 0x00, 0x85, 0x48, 0x04, 0x21, 0x20, 0xa2, 0x08, 0x22, 0x0a, 0x08, 0x22, 0x02, 0x08, 0x22, 0x0a, 0x08, 0x20, 0xa2, 0x21, 0x21, 0x04, 0x85, 0x48, 0x55, 0x05, 0x52, 0x00, 0x21, 0x54, 0xaa, 0x85, 0x55, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-           
//	 5	----------- 
//	 6	------ ---- 
//	 7	------ --- -
//	 8	------ -- --
//	 9	------   ---
//	10	------ -----
//	11	----- ------
//	12	----- ------
//	13	---- -------
//	14	---- -------
//	15	--- --------
//	16	-- ---------
//	17	------------
//		000000000011
//		012345678901
const char chr77[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x55, 0x56, 0x55, 0x59, 0x56, 0x55, 0x59, 0x59, 0x55, 0x59, 0x65, 0x55, 0x5a, 0x95, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	----------- 
//	 4	----------- 
//	 5	---------- -
//	 6	--------- --
//	 7	-------  ---
//	 8	-----   ----
//	 9	---  -- ----
//	10	-  ---- ----
//	11	------- ----
//	12	------- ----
//	13	------- ----
//	14	------- ----
//	15	------- ----
//	16	------- ----
//	17	------------
//		000000000011
//		012345678901
const char chr78[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x56, 0x95, 0x55, 0x6a, 0x55, 0x56, 0x96, 0x55, 0x69, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------ -----
//	 4	------ -----
//	 5	-           
//	 6	- --------- 
//	 7	- --------- 
//	 8	- --------- 
//	 9	----------- 
//	10	---------- -
//	11	---------- -
//	12	--------- --
//	13	--------- --
//	14	-------- ---
//	15	------- ----
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr79[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x6a, 0xaa, 0xaa, 0x65, 0x55, 0x56, 0x65, 0x55, 0x56, 0x65, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	--         -
//	 6	------ -----
//	 7	------ -----
//	 8	------ -----
//	 9	------ -----
//	10	------ -----
//	11	------ -----
//	12	------ -----
//	13	------ -----
//	14	------ -----
//	15	------ -----
//	16	-           
//	17	------------
//		000000000011
//		012345678901
const char chr80[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xaa, 0xa9, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------- ----
//	 4	------- ----
//	 5	------- ----
//	 6	-           
//	 7	------  ----
//	 8	----- - ----
//	 9	----- - ----
//	10	---- -- ----
//	11	---- -- ----
//	12	--- --- ----
//	13	-- ---- ----
//	14	- ----- ----
//	15	------- ----
//	16	------  ----
//	17	------------
//		000000000011
//		012345678901
const char chr81[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x5a, 0x55, 0x55, 0x66, 0x55, 0x55, 0x66, 0x55, 0x55, 0x96, 0x55, 0x55, 0x96, 0x55, 0x56, 0x56, 0x55, 0x59, 0x56, 0x55, 0x65, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------ -----
//	 4	------ -----
//	 5	------ -----
//	 6	-           
//	 7	------ ---- 
//	 8	------ ---- 
//	 9	----- ----- 
//	10	----- ----- 
//	11	----- ----- 
//	12	---- ------ 
//	13	---- ------ 
//	14	--- ------- 
//	15	--- ------- 
//	16	-- ------  -
//	17	------------
//		000000000011
//		012345678901
const char chr82[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x59, 0x56, 0x55, 0x59, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x56, 0x55, 0x95, 0x56, 0x55, 0x95, 0x56, 0x56, 0x55, 0x56, 0x56, 0x55, 0x56, 0x59, 0x55, 0x69, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	---- -------
//	 4	---- -------
//	 5	--        --
//	 6	----- ------
//	 7	----- ------
//	 8	----- ------
//	 9	------ -----
//	10	-           
//	11	------ -----
//	12	------- ----
//	13	------- ----
//	14	------- ----
//	15	-------- ---
//	16	-------- ---
//	17	------------
//		000000000011
//		012345678901
const char chr83[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x5a, 0xaa, 0xa5, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x59, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x59, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	----- ------
//	 4	----- ------
//	 5	----        
//	 6	---- ------ 
//	 7	--- ------- 
//	 8	-- -------- 
//	 9	- -------- -
//	10	---------- -
//	11	---------- -
//	12	--------- --
//	13	--------- --
//	14	-------- ---
//	15	-----   ----
//	16	--   -------
//	17	------------
//		000000000011
//		012345678901
const char chr84[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x95, 0x56, 0x56, 0x55, 0x56, 0x59, 0x55, 0x56, 0x65, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x6a, 0x55, 0x5a, 0x95, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-- ---------
//	 4	-- ---------
//	 5	--          
//	 6	-- ---- ----
//	 7	- ----- ----
//	 8	- ----- ----
//	 9	------- ----
//	10	------- ----
//	11	------- ----
//	12	------- ----
//	13	------ -----
//	14	------ -----
//	15	----- ------
//	16	---  -------
//	17	------------
//		000000000011
//		012345678901
const char chr85[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x5a, 0xaa, 0xaa, 0x59, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x56, 0x95, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	--         -
//	 6	---------- -
//	 7	---------- -
//	 8	---------- -
//	 9	---------- -
//	10	---------- -
//	11	---------- -
//	12	---------- -
//	13	---------- -
//	14	---------- -
//	15	---------- -
//	16	--         -
//	17	------------
//		000000000011
//		012345678901
const char chr86[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xaa, 0xa9, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x5a, 0xaa, 0xa9, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--- ----- --
//	 4	--- ----- --
//	 5	-           
//	 6	--- ----- --
//	 7	--- ----- --
//	 8	--- ----- --
//	 9	--------- --
//	10	--------- --
//	11	--------- --
//	12	--------- --
//	13	-------- ---
//	14	-------- ---
//	15	------  ----
//	16	----  ------
//	17	------------
//		000000000011
//		012345678901
const char chr87[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x6a, 0xaa, 0xaa, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x56, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-    -------
//	 5	----- ------
//	 6	------------
//	 7	----------- 
//	 8	-    ------ 
//	 9	----- ----- 
//	10	----------- 
//	11	----------- 
//	12	---------- -
//	13	---------- -
//	14	--------- --
//	15	-------  ---
//	16	-      -----
//	17	------------
//		000000000011
//		012345678901
const char chr88[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0x95, 0x55, 0x55, 0x65, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x6a, 0x95, 0x56, 0x55, 0x65, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x56, 0x95, 0x6a, 0xa9, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-          -
//	 6	---------- -
//	 7	--------- --
//	 8	--------- --
//	 9	-------- ---
//	10	-------- ---
//	11	------- ----
//	12	------  ----
//	13	----- -- ---
//	14	---- ---- --
//	15	--- ------ -
//	16	-  -------- 
//	17	------------
//		000000000011
//		012345678901
const char chr89[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xa9, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x65, 0x95, 0x55, 0x95, 0x65, 0x56, 0x55, 0x59, 0x69, 0x55, 0x56, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--- --------
//	 4	--- --------
//	 5	--- --------
//	 6	--- --------
//	 7	--- -       
//	 8	-    ------ 
//	 9	--- ------ -
//	10	--- ----- --
//	11	--- --------
//	12	--- --------
//	13	--- --------
//	14	--- --------
//	15	---- -------
//	16	-----      -
//	17	------------
//		000000000011
//		012345678901
const char chr90[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x6a, 0xaa, 0x6a, 0x95, 0x56, 0x56, 0x55, 0x59, 0x56, 0x55, 0x65, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x6a, 0xa9, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	----------- 
//	 5	- --------- 
//	 6	-- -------- 
//	 7	-- -------- 
//	 8	--- ------- 
//	 9	---- ------ 
//	10	---------- -
//	11	---------- -
//	12	--------- --
//	13	-------- ---
//	14	------- ----
//	15	-----  -----
//	16	--   -------
//	17	------------
//		000000000011
//		012345678901
const char chr91[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x65, 0x55, 0x56, 0x59, 0x55, 0x56, 0x59, 0x55, 0x56, 0x56, 0x55, 0x56, 0x55, 0x95, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x69, 0x55, 0x5a, 0x95, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	----- ------
//	 4	----- ------
//	 5	-----       
//	 6	---- ------ 
//	 7	---- ------ 
//	 8	--- ------- 
//	 9	-  -- ---- -
//	10	------  -- -
//	11	--------  --
//	12	--------- --
//	13	-------- ---
//	14	------  ----
//	15	----  ------
//	16	-   --------
//	17	------------
//		000000000011
//		012345678901
const char chr92[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x6a, 0xaa, 0x55, 0x95, 0x56, 0x55, 0x95, 0x56, 0x56, 0x55, 0x56, 0x69, 0x65, 0x59, 0x55, 0x5a, 0x59, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0xa5, 0x55, 0x6a, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--------- --
//	 4	-------  ---
//	 5	-----  -----
//	 6	---  -- ----
//	 7	------- ----
//	 8	------- ----
//	 9	-           
//	10	------- ----
//	11	------- ----
//	12	------ -----
//	13	------ -----
//	14	----- ------
//	15	---- -------
//	16	--  --------
//	17	------------
//		000000000011
//		012345678901
const char chr93[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x55, 0x56, 0x95, 0x55, 0x69, 0x55, 0x56, 0x96, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	- -- ----- -
//	 5	- -- ----- -
//	 6	-- -- ---- -
//	 7	-- -- ---- -
//	 8	-- -- ---- -
//	 9	---------- -
//	10	--------- --
//	11	--------- --
//	12	-------- ---
//	13	-------- ---
//	14	------- ----
//	15	------ -----
//	16	----  ------
//	17	------------
//		000000000011
//		012345678901
const char chr94[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x95, 0x59, 0x65, 0x95, 0x59, 0x59, 0x65, 0x59, 0x59, 0x65, 0x59, 0x59, 0x65, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---       --
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	-           
//	 9	------ -----
//	10	------ -----
//	11	------ -----
//	12	------ -----
//	13	----- ------
//	14	----- ------
//	15	---- -------
//	16	--  --------
//	17	------------
//		000000000011
//		012345678901
const char chr95[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0xaa, 0xa5, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--- --------
//	 4	--- --------
//	 5	--- --------
//	 6	--- --------
//	 7	--- --------
//	 8	---  -------
//	 9	--- -  -----
//	10	--- --- ----
//	11	--- ---- ---
//	12	--- ----- --
//	13	--- --------
//	14	--- --------
//	15	--- --------
//	16	--- --------
//	17	------------
//		000000000011
//		012345678901
const char chr96[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x95, 0x55, 0x56, 0x69, 0x55, 0x56, 0x56, 0x55, 0x56, 0x55, 0x95, 0x56, 0x55, 0x65, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------- ----
//	 4	------- ----
//	 5	------- ----
//	 6	-           
//	 7	------- ----
//	 8	------- ----
//	 9	------- ----
//	10	------- ----
//	11	------- ----
//	12	------ -----
//	13	------ -----
//	14	----- ------
//	15	---- -------
//	16	--  --------
//	17	------------
//		000000000011
//		012345678901
const char chr97[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---       --
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	-           
//	17	------------
//		000000000011
//		012345678901
const char chr98[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0xaa, 0xa5, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---        -
//	 6	---------- -
//	 7	---------- -
//	 8	---------- -
//	 9	---- ---- --
//	10	----- --- --
//	11	------ - ---
//	12	------- ----
//	13	------- ----
//	14	------ - ---
//	15	----  --- --
//	16	--  ------ -
//	17	------------
//		000000000011
//		012345678901
const char chr99[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0xaa, 0xa9, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x95, 0x65, 0x55, 0x65, 0x65, 0x55, 0x59, 0x95, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x95, 0x55, 0xa5, 0x65, 0x5a, 0x55, 0x59, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----  -----
//	 4	------- ----
//	 5	------------
//	 6	--         -
//	 7	--------- --
//	 8	-------- ---
//	 9	------- ----
//	10	------  ----
//	11	-----  -  --
//	12	---  - --- -
//	13	-  --- ---- 
//	14	------ -----
//	15	------ -----
//	16	------ -----
//	17	------------
//		000000000011
//		012345678901
const char chr100[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xaa, 0xa9, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x69, 0xa5, 0x56, 0x99, 0x59, 0x69, 0x59, 0x56, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---------- -
//	 5	---------- -
//	 6	---------- -
//	 7	---------- -
//	 8	--------- --
//	 9	--------- --
//	10	-------- ---
//	11	-------- ---
//	12	------- ----
//	13	------ -----
//	14	----- ------
//	15	---- -------
//	16	--  --------
//	17	------------
//		000000000011
//		012345678901
const char chr101[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x5a, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	----- -- ---
//	 5	----- -- ---
//	 6	----- --- --
//	 7	---- ---- --
//	 8	---- ---- --
//	 9	---- ---- --
//	10	--- ------ -
//	11	--- ------ -
//	12	--- ------ -
//	13	-- -------- 
//	14	-- -------- 
//	15	-- -------- 
//	16	- --------- 
//	17	------------
//		000000000011
//		012345678901
const char chr102[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x95, 0x55, 0x65, 0x95, 0x55, 0x65, 0x65, 0x55, 0x95, 0x65, 0x55, 0x95, 0x65, 0x55, 0x95, 0x65, 0x56, 0x55, 0x59, 0x56, 0x55, 0x59, 0x56, 0x55, 0x59, 0x59, 0x55, 0x56, 0x59, 0x55, 0x56, 0x59, 0x55, 0x56, 0x65, 0x55, 0x56, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	- ----------
//	 4	- ----------
//	 5	- ----------
//	 6	- ----------
//	 7	- ------   -
//	 8	-       ----
//	 9	- ----------
//	10	- ----------
//	11	- ----------
//	12	- ----------
//	13	- ----------
//	14	- ----------
//	15	-- -------- 
//	16	---        -
//	17	------------
//		000000000011
//		012345678901
const char chr103[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0xa9, 0x6a, 0xaa, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x59, 0x55, 0x56, 0x56, 0xaa, 0xa9, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-          -
//	 6	---------- -
//	 7	---------- -
//	 8	---------- -
//	 9	---------- -
//	10	--------- --
//	11	--------- --
//	12	-------- ---
//	13	-------- ---
//	14	------- ----
//	15	-----  -----
//	16	---  -------
//	17	------------
//		000000000011
//		012345678901
const char chr104[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xa9, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x69, 0x55, 0x56, 0x95, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---- -------
//	 6	----  ------
//	 7	--- - ------
//	 8	--- -- -----
//	 9	-- ---- ----
//	10	-- ---- ----
//	11	- ------ ---
//	12	--------- --
//	13	--------- --
//	14	---------- -
//	15	----------- 
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr105[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0xa5, 0x55, 0x56, 0x65, 0x55, 0x56, 0x59, 0x55, 0x59, 0x56, 0x55, 0x59, 0x56, 0x55, 0x65, 0x55, 0x95, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x59, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------ -----
//	 3	------ -----
//	 4	-           
//	 5	------ -----
//	 6	------ -----
//	 7	------ -----
//	 8	------ -----
//	 9	--- -- -- --
//	10	--- -- -- --
//	11	-- --- --- -
//	12	-- --- --- -
//	13	-- --- --- -
//	14	- ---- ---- 
//	15	------ -----
//	16	------ -----
//	17	------------
//		000000000011
//		012345678901
const char chr106[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x56, 0x59, 0x65, 0x56, 0x59, 0x65, 0x59, 0x59, 0x59, 0x59, 0x59, 0x59, 0x59, 0x59, 0x59, 0x65, 0x59, 0x56, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-           
//	 6	----------- 
//	 7	----------- 
//	 8	---------- -
//	 9	---------- -
//	10	--- ----- --
//	11	---- --- ---
//	12	-----   ----
//	13	------ -----
//	14	------- ----
//	15	-------- ---
//	16	--------- --
//	17	------------
//		000000000011
//		012345678901
const char chr107[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x56, 0x55, 0x65, 0x55, 0x95, 0x95, 0x55, 0x6a, 0x55, 0x55, 0x59, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x65, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	--    ------
//	 6	------  ----
//	 7	--------  --
//	 8	------------
//	 9	--    ------
//	10	------  ----
//	11	--------  --
//	12	------------
//	13	------------
//	14	--    ------
//	15	------  ----
//	16	--------  --
//	17	------------
//		000000000011
//		012345678901
const char chr108[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xa5, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x55, 0x5a, 0xa5, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xa5, 0x55, 0x55, 0x5a, 0x55, 0x55, 0x55, 0xa5, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------ -----
//	 5	------ -----
//	 6	----- ------
//	 7	----- ------
//	 8	---- -------
//	 9	---- -------
//	10	---- -- ----
//	11	--- ---- ---
//	12	--- ---- ---
//	13	-- ------ --
//	14	-- ---    --
//	15	-     ---- -
//	16	----------- 
//	17	------------
//		000000000011
//		012345678901
const char chr109[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x55, 0x96, 0x55, 0x56, 0x55, 0x95, 0x56, 0x55, 0x95, 0x59, 0x55, 0x65, 0x59, 0x5a, 0xa5, 0x6a, 0xa5, 0x59, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	---------- -
//	 4	---------- -
//	 5	---------- -
//	 6	--------- --
//	 7	---- ---- --
//	 8	-----  -- --
//	 9	-------  ---
//	10	-------- ---
//	11	------- - --
//	12	------- -- -
//	13	------ ---- 
//	14	----- ------
//	15	---  -------
//	16	-  ---------
//	17	------------
//		000000000011
//		012345678901
const char chr110[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x95, 0x65, 0x55, 0x69, 0x65, 0x55, 0x56, 0x95, 0x55, 0x55, 0x95, 0x55, 0x56, 0x65, 0x55, 0x56, 0x59, 0x55, 0x59, 0x56, 0x55, 0x65, 0x55, 0x56, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	--         -
//	 5	----- ------
//	 6	----- ------
//	 7	----- ------
//	 8	----- ------
//	 9	-           
//	10	----- ------
//	11	----- ------
//	12	----- ------
//	13	----- ------
//	14	----- ------
//	15	----- ------
//	16	------     -
//	17	------------
//		000000000011
//		012345678901
const char chr111[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xaa, 0xa9, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x6a, 0xaa, 0xaa, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x5a, 0xa9, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	-----##-----
//	12	----#  #----
//	13	---#    #---
//	14	--#      #--
//	15	-#        #-
//	16	#          #
//	17	############
//		000000000011
//		012345678901
const char chr112[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-----##-----
//	 6	----#  #----
//	 7	---#    #---
//	 8	--#      #--
//	 9	-#        #-
//	10	#          #
//	11	############
//	12	----#  #----
//	13	---#    #---
//	14	--#      #--
//	15	-#        #-
//	16	#          #
//	17	############
//		000000000011
//		012345678901
const char chr113[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00};


//	 0	----#  #----
//	 1	---#    #---
//	 2	--#      #--
//	 3	-#        #-
//	 4	#          #
//	 5	############
//	 6	----#  #----
//	 7	---#    #---
//	 8	--#      #--
//	 9	-#        #-
//	10	#          #
//	11	############
//	12	----#  #----
//	13	---#    #---
//	14	--#      #--
//	15	-#        #-
//	16	#          #
//	17	############
//		000000000011
//		012345678901
const char chr114[] PROGMEM = {0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00};


//	 0	----####----
//	 1	---######---
//	 2	--########--
//	 3	-##########-
//	 4	############
//	 5	############
//	 6	----####----
//	 7	---######---
//	 8	--########--
//	 9	-##########-
//	10	############
//	11	############
//	12	----####----
//	13	---######---
//	14	--########--
//	15	-##########-
//	16	############
//	17	############
//		000000000011
//		012345678901
const char chr115[] PROGMEM = {0x55, 0x00, 0x55, 0x54, 0x00, 0x15, 0x50, 0x00, 0x05, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0x00, 0x55, 0x54, 0x00, 0x15, 0x50, 0x00, 0x05, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x55, 0x00, 0x55, 0x54, 0x00, 0x15, 0x50, 0x00, 0x05, 0x40, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};


//	 0	############
//	 1	############
//	 2	-##########-
//	 3	--########--
//	 4	---######---
//	 5	----####----
//	 6	############
//	 7	############
//	 8	-##########-
//	 9	--########--
//	10	---######---
//	11	----####----
//	12	############
//	13	############
//	14	-##########-
//	15	--########--
//	16	---######---
//	17	----####----
//		000000000011
//		012345678901
const char chr116[] PROGMEM = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x01, 0x50, 0x00, 0x05, 0x54, 0x00, 0x15, 0x55, 0x00, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x01, 0x50, 0x00, 0x05, 0x54, 0x00, 0x15, 0x55, 0x00, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x01, 0x50, 0x00, 0x05, 0x54, 0x00, 0x15, 0x55, 0x00, 0x55};


//	 0	############
//	 1	#          #
//	 2	-#        #-
//	 3	--#      #--
//	 4	---#    #---
//	 5	----#  #----
//	 6	############
//	 7	#          #
//	 8	-#        #-
//	 9	--#      #--
//	10	---#    #---
//	11	----#  #----
//	12	############
//	13	#          #
//	14	-#        #-
//	15	--#      #--
//	16	---#    #---
//	17	----#  #----
//		000000000011
//		012345678901
const char chr117[] PROGMEM = {0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55};


//	 0	############
//	 1	#          #
//	 2	-#        #-
//	 3	--#      #--
//	 4	---#    #---
//	 5	----#  #----
//	 6	############
//	 7	#          #
//	 8	-#        #-
//	 9	--#      #--
//	10	---#    #---
//	11	----#  #----
//	12	-----##-----
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr118[] PROGMEM = {0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	############
//	 1	#          #
//	 2	-#        #-
//	 3	--#      #--
//	 4	---#    #---
//	 5	----#  #----
//	 6	-----##-----
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr119[] PROGMEM = {0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	-##########-
//	 1	#          #
//	 2	#          #
//	 3	-##########-
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr120[] PROGMEM = {0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-##########-
//	 5	#          #
//	 6	#          #
//	 7	-##########-
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr121[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	-##########-
//	 8	#          #
//	 9	#          #
//	10	-##########-
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr122[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	-##########-
//	11	#          #
//	12	#          #
//	13	-##########-
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr123[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	-##########-
//	15	#          #
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr124[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	#######-----
//	 4	#     #-----
//	 5	# # # #-----
//	 6	# # # #-----
//	 7	########----
//	 8	----#  #----
//	 9	----# # #---
//	10	----#  #----
//	11	----# ###---
//	12	----# ## #--
//	13	----#### ###
//	14	-------#   #
//	15	-------# # #
//	16	-------# # #
//	17	-------##-##
//		000000000011
//		012345678901
const char chr125[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x01, 0x55, 0x2a, 0xa1, 0x55, 0x22, 0x21, 0x55, 0x22, 0x21, 0x55, 0x00, 0x00, 0x55, 0x55, 0x28, 0x55, 0x55, 0x22, 0x15, 0x55, 0x28, 0x55, 0x55, 0x20, 0x15, 0x55, 0x20, 0x85, 0x55, 0x00, 0x80, 0x55, 0x54, 0xa8, 0x55, 0x54, 0x88, 0x55, 0x54, 0x88, 0x55, 0x54, 0x10};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	--##-###----
//	12	-#  -# #----
//	13	# ###   #---
//	14	#  #-# #----
//	15	# #--# #----
//	16	# #--# #----
//	17	-#----#-----
//		000000000011
//		012345678901
const char chr126[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x40, 0x55, 0x4a, 0x48, 0x55, 0x20, 0x2a, 0x15, 0x28, 0x48, 0x55, 0x21, 0x48, 0x55, 0x21, 0x48, 0x55, 0x45, 0x51, 0x55};


//	 0	---------- -
//	 1	-------- -- 
//	 2	--------- --
//	 3	----- ------
//	 4	----- ------
//	 5	----        
//	 6	---- ------ 
//	 7	--- ------- 
//	 8	-- -------- 
//	 9	- -------- -
//	10	---------- -
//	11	---------- -
//	12	--------- --
//	13	--------- --
//	14	-------- ---
//	15	-----   ----
//	16	--   -------
//	17	------------
//		000000000011
//		012345678901
const char chr127[] PROGMEM = {0x55, 0x55, 0x59, 0x55, 0x55, 0x96, 0x55, 0x55, 0x65, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x95, 0x56, 0x56, 0x55, 0x56, 0x59, 0x55, 0x56, 0x65, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x6a, 0x55, 0x5a, 0x95, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-#----------
//	 2	# #---------
//	 3	# #---------
//	 4	# ###-------
//	 5	# # #-------
//	 6	#  #####----
//	 7	# # #  #----
//	 8	##-## # #---
//	 9	----#  #----
//	10	----# ###---
//	11	----# ## #--
//	12	----#### ###
//	13	-------#   #
//	14	-------# # #
//	15	-------# # #
//	16	-------##-##
//	17	------------
//		000000000011
//		012345678901
const char chr128[] PROGMEM = {0x55, 0x55, 0x55, 0x45, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x20, 0x15, 0x55, 0x22, 0x15, 0x55, 0x28, 0x00, 0x55, 0x22, 0x28, 0x55, 0x04, 0x22, 0x15, 0x55, 0x28, 0x55, 0x55, 0x20, 0x15, 0x55, 0x20, 0x85, 0x55, 0x00, 0x80, 0x55, 0x54, 0xa8, 0x55, 0x54, 0x88, 0x55, 0x54, 0x88, 0x55, 0x54, 0x10, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	--  ---- ---
//	 9	- ## -- # --
//	10	 #  #  # # -
//	11	 #  # # # --
//	12	- ## # # ---
//	13	--  # #  ---
//	14	-- # # ## --
//	15	- # # #  # -
//	16	 # #  #  # -
//	17	- # -- ## --
//		000000000011
//		012345678901
const char chr129[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0x55, 0x95, 0x60, 0x96, 0x25, 0x8a, 0x28, 0x89, 0x8a, 0x22, 0x25, 0x60, 0x88, 0x95, 0x5a, 0x22, 0x95, 0x58, 0x88, 0x25, 0x62, 0x22, 0x89, 0x88, 0xa2, 0x89, 0x62, 0x58, 0x25};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	# # # # ####
//	 7	 ------#   #
//	 8	#-------#  #
//	 9	 ------# # #
//	10	#-----# #-# 
//	11	 ----# #---#
//	12	#---# #---- 
//	13	 #-# #-----#
//	14	# # #------ 
//	15	#  #-------#
//	16	#   #------ 
//	17	#### # # # #
//		000000000011
//		012345678901
const char chr130[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x22, 0x22, 0x00, 0x95, 0x54, 0xa8, 0x15, 0x55, 0x28, 0x95, 0x54, 0x88, 0x15, 0x52, 0x12, 0x95, 0x48, 0x54, 0x15, 0x21, 0x56, 0x84, 0x85, 0x54, 0x22, 0x15, 0x56, 0x28, 0x55, 0x54, 0x2a, 0x15, 0x56, 0x00, 0x88, 0x88};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	----- ------
//	 6	---- # -----
//	 7	--- # # ----
//	 8	-- #   # ---
//	 9	- #     # --
//	10	-- #   # ---
//	11	--- # # ----
//	12	---- # -----
//	13	----- ------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr131[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x65, 0x55, 0x55, 0x89, 0x55, 0x56, 0x22, 0x55, 0x58, 0xa8, 0x95, 0x62, 0xaa, 0x25, 0x58, 0xa8, 0x95, 0x56, 0x22, 0x55, 0x55, 0x89, 0x55, 0x55, 0x65, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	-#######----
//	 1	#       #---
//	 2	# ## ## #---
//	 3	# ## ## #---
//	 4	# #-#-# #---
//	 5	-#-###-#----
//	 6	-##   ##----
//	 7	-# ### #----
//	 8	-#     #----
//	 9	-# ### #----
//	10	-# #-# #----
//	11	--#---#-----
//	12	-# #--------
//	13	-# #####----
//	14	-#     #----
//	15	-# ### #----
//	16	-# #-# #----
//	17	--#---#-----
//		000000000011
//		012345678901
const char chr132[] PROGMEM = {0x40, 0x00, 0x55, 0x2a, 0xaa, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x21, 0x12, 0x15, 0x44, 0x04, 0x55, 0x42, 0xa0, 0x55, 0x48, 0x08, 0x55, 0x4a, 0xa8, 0x55, 0x48, 0x08, 0x55, 0x48, 0x48, 0x55, 0x51, 0x51, 0x55, 0x48, 0x55, 0x55, 0x48, 0x00, 0x55, 0x4a, 0xa8, 0x55, 0x48, 0x08, 0x55, 0x48, 0x48, 0x55, 0x51, 0x51, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#       ## #
//	 7	#  ##  ##  #
//	 8	    ####    
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr133[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0x08, 0x28, 0x28, 0x28, 0xaa, 0x00, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	            
//	 2	############
//	 3	##   ##  ###
//	 4	#### ## # ##
//	 5	###  ## # ##
//	 6	#### ## # ##
//	 7	##   ##  ###
//	 8	############
//	 9	##   # #####
//	10	## #########
//	11	##  ## # # #
//	12	## ### ## ##
//	13	## ### # # #
//	14	############
//	15	            
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr134[] PROGMEM = {0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x0a, 0x82, 0x80, 0x00, 0x82, 0x20, 0x02, 0x82, 0x20, 0x00, 0x82, 0x20, 0x0a, 0x82, 0x80, 0x00, 0x00, 0x00, 0x0a, 0x88, 0x00, 0x08, 0x00, 0x00, 0x0a, 0x08, 0x88, 0x08, 0x08, 0x20, 0x08, 0x08, 0x88, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	--##--------
//	 1	-#  #-------
//	 2	# ## #------
//	 3	# ## #------
//	 4	#    #------
//	 5	# ## #------
//	 6	# ## #------
//	 7	-#--#-------
//	 8	---# #------
//	 9	---# #------
//	10	---# #------
//	11	---# ##-###-
//	12	---#   #   #
//	13	----###-# #-
//	14	--------# #-
//	15	--------# #-
//	16	--------# #-
//	17	---------#--
//		000000000011
//		012345678901
const char chr135[] PROGMEM = {0x50, 0x55, 0x55, 0x4a, 0x15, 0x55, 0x20, 0x85, 0x55, 0x20, 0x85, 0x55, 0x2a, 0x85, 0x55, 0x20, 0x85, 0x55, 0x20, 0x85, 0x55, 0x45, 0x15, 0x55, 0x54, 0x85, 0x55, 0x54, 0x85, 0x55, 0x54, 0x85, 0x55, 0x54, 0x81, 0x01, 0x54, 0xa8, 0xa8, 0x55, 0x01, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x45};


//	 0	-#---#------
//	 1	# #-# #-----
//	 2	# #-# #-----
//	 3	# #-# #-----
//	 4	# #-# #-----
//	 5	-# # #------
//	 6	--# ####----
//	 7	---#    #---
//	 8	---# ###----
//	 9	---#   #----
//	10	---# ##-#---
//	11	---# ### #--
//	12	---#     #--
//	13	----#### #--
//	14	-------# #--
//	15	-------# ###
//	16	-------#    
//	17	--------####
//		000000000011
//		012345678901
const char chr136[] PROGMEM = {0x45, 0x45, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x48, 0x85, 0x55, 0x52, 0x00, 0x55, 0x54, 0xaa, 0x15, 0x54, 0x80, 0x55, 0x54, 0xa8, 0x55, 0x54, 0x81, 0x15, 0x54, 0x80, 0x85, 0x54, 0xaa, 0x85, 0x55, 0x00, 0x85, 0x55, 0x54, 0x85, 0x55, 0x54, 0x80, 0x55, 0x54, 0xaa, 0x55, 0x55, 0x00};


//	 0	------------
//	 1	-####-------
//	 2	#   ##------
//	 3	# ## #------
//	 4	# ## #------
//	 5	# ## #------
//	 6	#   ####----
//	 7	-###    #---
//	 8	---# ###----
//	 9	---#    #---
//	10	---#### #---
//	11	---#    ###-
//	12	----##     #
//	13	------## ##-
//	14	-------# #--
//	15	-------# #--
//	16	-------# #--
//	17	--------#---
//		000000000011
//		012345678901
const char chr137[] PROGMEM = {0x55, 0x55, 0x55, 0x40, 0x15, 0x55, 0x2a, 0x05, 0x55, 0x20, 0x85, 0x55, 0x20, 0x85, 0x55, 0x20, 0x85, 0x55, 0x2a, 0x00, 0x55, 0x40, 0xaa, 0x15, 0x54, 0x80, 0x55, 0x54, 0xaa, 0x15, 0x54, 0x02, 0x15, 0x54, 0xaa, 0x01, 0x55, 0x0a, 0xa8, 0x55, 0x50, 0x81, 0x55, 0x54, 0x85, 0x55, 0x54, 0x85, 0x55, 0x54, 0x85, 0x55, 0x55, 0x15};


//	 0	-#######----
//	 1	#       #---
//	 2	# ## ## #---
//	 3	# ## ## #---
//	 4	# ## ## #---
//	 5	# #-#-# #---
//	 6	-#-----#----
//	 7	-#######----
//	 8	#       #---
//	 9	#       #---
//	10	-#######----
//	11	--#####-----
//	12	-#     #----
//	13	-# #####----
//	14	-#     #----
//	15	-##### #----
//	16	-#     #----
//	17	--#####-----
//		000000000011
//		012345678901
const char chr138[] PROGMEM = {0x40, 0x00, 0x55, 0x2a, 0xaa, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x21, 0x12, 0x15, 0x45, 0x54, 0x55, 0x40, 0x00, 0x55, 0x2a, 0xaa, 0x15, 0x2a, 0xaa, 0x15, 0x40, 0x00, 0x55, 0x50, 0x01, 0x55, 0x4a, 0xa8, 0x55, 0x48, 0x00, 0x55, 0x4a, 0xa8, 0x55, 0x40, 0x08, 0x55, 0x4a, 0xa8, 0x55, 0x50, 0x01, 0x55};


//	 0	--   -------
//	 1	- ### ------
//	 2	 ## ## -----
//	 3	 # - # -----
//	 4	 ## ## -----
//	 5	- ### ------
//	 6	--   -------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr139[] PROGMEM = {0x5a, 0x95, 0x55, 0x60, 0x25, 0x55, 0x82, 0x09, 0x55, 0x89, 0x89, 0x55, 0x82, 0x09, 0x55, 0x60, 0x25, 0x55, 0x5a, 0x95, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	-#######----
//	12	#       #---
//	13	# ## ## #---
//	14	# ## ## #---
//	15	# ## ## #---
//	16	# #-#-# #---
//	17	-#-----#----
//		000000000011
//		012345678901
const char chr140[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x00, 0x55, 0x2a, 0xaa, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x21, 0x12, 0x15, 0x45, 0x54, 0x55};


//	 0	############
//	 1	-- # -------
//	 2	- # # ------
//	 3	 #   #     -
//	 4	### ###### -
//	 5	  # #  # # -
//	 6	- # #  # #  
//	 7	- # #  # ###
//	 8	- # #  #   #
//	 9	- # #  # # #
//	10	- # #  # # #
//	11	- # #  # # #
//	12	  # #  #####
//	13	### ###     
//	14	 #   # -----
//	15	- # # ------
//	16	-- # -------
//	17	############
//		000000000011
//		012345678901
const char chr141[] PROGMEM = {0x00, 0x00, 0x00, 0x58, 0x95, 0x55, 0x62, 0x25, 0x55, 0x8a, 0x8a, 0xa9, 0x02, 0x00, 0x09, 0xa2, 0x28, 0x89, 0x62, 0x28, 0x8a, 0x62, 0x28, 0x80, 0x62, 0x28, 0xa8, 0x62, 0x28, 0x88, 0x62, 0x28, 0x88, 0x62, 0x28, 0x88, 0xa2, 0x28, 0x00, 0x02, 0x02, 0xaa, 0x8a, 0x89, 0x55, 0x62, 0x25, 0x55, 0x58, 0x95, 0x55, 0x00, 0x00, 0x00};


//	 0	------- ----
//	 1	------ # ---
//	 2	------ # ---
//	 3	----   # ---
//	 4	--- #### ---
//	 5	-- #   # ---
//	 6	-- # - # ---
//	 7	-- #   # ---
//	 8	--- #### ---
//	 9	----    ----
//	10	- -------- -
//	11	 # ------ # 
//	12	 # -#--#- # 
//	13	 # # ## # # 
//	14	 ##      ## 
//	15	 # # ## # # 
//	16	 # -#--#- # 
//	17	- -------- -
//		000000000011
//		012345678901
const char chr142[] PROGMEM = {0x55, 0x56, 0x55, 0x55, 0x58, 0x95, 0x55, 0x58, 0x95, 0x55, 0xa8, 0x95, 0x56, 0x00, 0x95, 0x58, 0xa8, 0x95, 0x58, 0x98, 0x95, 0x58, 0xa8, 0x95, 0x56, 0x00, 0x95, 0x55, 0xaa, 0x55, 0x65, 0x55, 0x59, 0x89, 0x55, 0x62, 0x89, 0x14, 0x62, 0x88, 0x82, 0x22, 0x82, 0xaa, 0x82, 0x88, 0x82, 0x22, 0x89, 0x14, 0x62, 0x65, 0x55, 0x59};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-------- ---
//	 4	--------- --
//	 5	-------- ---
//	 6	------- ----
//	 7	-----  -----
//	 8	---  -------
//	 9	-- ---------
//	10	-- ---------
//	11	---  -------
//	12	-----  -----
//	13	-------  ---
//	14	--------- --
//	15	---------- -
//	16	---------- -
//	17	------------
//		000000000011
//		012345678901
const char chr143[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x65, 0x55, 0x55, 0x95, 0x55, 0x56, 0x55, 0x55, 0x69, 0x55, 0x56, 0x95, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x56, 0x95, 0x55, 0x55, 0x69, 0x55, 0x55, 0x56, 0x95, 0x55, 0x55, 0x65, 0x55, 0x55, 0x59, 0x55, 0x55, 0x59, 0x55, 0x55, 0x55};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-# ###### #-
//	 4	-# ###### #-
//	 5	-# ###### #-
//	 6	-# ###### #-
//	 7	-# ###### #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr144[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-# ###### #-
//	 5	-# ###### #-
//	 6	-# ###### #-
//	 7	-# ###### #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr145[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-# ###### #-
//	 6	-# ###### #-
//	 7	-# ###### #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr146[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-# ###### #-
//	 7	-# ###### #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr147[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-# ###### #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr148[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-# ###### #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr149[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-# ###### #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr150[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-# ###### #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr151[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-# ###### #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr152[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-#        #-
//	12	-# ###### #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr153[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-#        #-
//	12	-#        #-
//	13	-# ###### #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr154[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-#        #-
//	12	-#        #-
//	13	-#        #-
//	14	-# ###### #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr155[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-#        #-
//	12	-#        #-
//	13	-#        #-
//	14	-#        #-
//	15	-# ###### #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr156[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x00, 0x21, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	---######---
//	 1	-###    ###-
//	 2	-#   ##   #-
//	 3	-#   ##   #-
//	 4	-#        #-
//	 5	-#        #-
//	 6	-#        #-
//	 7	-#        #-
//	 8	-#        #-
//	 9	-#        #-
//	10	-#        #-
//	11	-#        #-
//	12	-#        #-
//	13	-#        #-
//	14	-#        #-
//	15	-#        #-
//	16	-#        #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr157[] PROGMEM = {0x54, 0x00, 0x15, 0x40, 0xaa, 0x01, 0x4a, 0x82, 0xa1, 0x4a, 0x82, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	-#---#------
//	11	# #-# #-----
//	12	# #-# #-----
//	13	# #-# #-----
//	14	# #-# #-----
//	15	## # ##-----
//	16	-## ##------
//	17	--###-------
//		000000000011
//		012345678901
const char chr158[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x45, 0x45, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x21, 0x21, 0x55, 0x08, 0x81, 0x55, 0x42, 0x05, 0x55, 0x50, 0x15, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	--###-------
//	11	-#   #------
//	12	# ### #-----
//	13	# ### #-----
//	14	#     #-----
//	15	# ### #-----
//	16	# #-# #-----
//	17	-#---#------
//		000000000011
//		012345678901
const char chr159[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x20, 0x21, 0x55, 0x20, 0x21, 0x55, 0x2a, 0xa1, 0x55, 0x20, 0x21, 0x55, 0x21, 0x21, 0x55, 0x45, 0x45, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #  ##  # -
//	 5	 #   ##   # 
//	 6	#    ##    #
//	 7	#    ##    #
//	 8	     ##     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr160[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0x82, 0x89, 0x8a, 0x82, 0xa2, 0x2a, 0x82, 0xa8, 0x2a, 0x82, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #     ## -
//	 5	 #     ## # 
//	 6	#     ##   #
//	 7	#     ##   #
//	 8	     ##     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr161[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x09, 0x8a, 0xa8, 0x22, 0x2a, 0xa0, 0xa8, 0x2a, 0xa0, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #      ### 
//	 6	#      ### #
//	 7	#     ###  #
//	 8	     ###    
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr162[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0x02, 0x2a, 0xa8, 0x08, 0x2a, 0xa0, 0x28, 0xaa, 0x80, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#        ###
//	 7	#      ### #
//	 8	     ####   
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr163[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0x80, 0x2a, 0xa8, 0x08, 0xaa, 0x80, 0x2a, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ###### 
//	 9	     ###### 
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr164[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x80, 0x02, 0xaa, 0x80, 0x02, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ####   
//	10	#      ### #
//	11	#        ###
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr165[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x80, 0x2a, 0x2a, 0xa8, 0x08, 0x2a, 0xaa, 0x80, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ###    
//	10	#     ###  #
//	11	#      ### #
//	12	 #      ### 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr166[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x80, 0xaa, 0x2a, 0xa0, 0x28, 0x2a, 0xa8, 0x08, 0x8a, 0xaa, 0x02, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ##     
//	10	#     ##   #
//	11	#     ##   #
//	12	 #     ## # 
//	13	- #     ## -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr167[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xa0, 0xa8, 0x2a, 0xa0, 0xa8, 0x8a, 0xa8, 0x22, 0x62, 0xaa, 0x09, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ##     
//	10	#    ##    #
//	11	#    ##    #
//	12	 #   ##   # 
//	13	- #  ##  # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr168[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0x82, 0xa8, 0x2a, 0x82, 0xa8, 0x8a, 0x82, 0xa2, 0x62, 0x82, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ##     
//	10	#   ##     #
//	11	#   ##     #
//	12	 # ##     # 
//	13	- ##     # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr169[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0x0a, 0xa8, 0x2a, 0x0a, 0xa8, 0x88, 0x2a, 0xa2, 0x60, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	    ###     
//	10	#  ###     #
//	11	# ###      #
//	12	 ###      # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr170[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x02, 0xaa, 0x28, 0x0a, 0xa8, 0x20, 0x2a, 0xa8, 0x80, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	   ####     
//	10	# ###      #
//	11	###        #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr171[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xa8, 0x02, 0xaa, 0x20, 0x2a, 0xa8, 0x02, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	 ######     
//	 9	 ######     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr172[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x80, 0x02, 0xaa, 0x80, 0x02, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	###        #
//	 7	# ###      #
//	 8	   ####     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr173[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x02, 0xaa, 0xa8, 0x20, 0x2a, 0xa8, 0xa8, 0x02, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 ###      # 
//	 6	# ###      #
//	 7	#  ###     #
//	 8	    ###     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr174[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x80, 0xaa, 0xa2, 0x20, 0x2a, 0xa8, 0x28, 0x0a, 0xa8, 0xaa, 0x02, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- ##     # -
//	 5	 # ##     # 
//	 6	#   ##     #
//	 7	#   ##     #
//	 8	     ##     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr175[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x60, 0xaa, 0x89, 0x88, 0x2a, 0xa2, 0x2a, 0x0a, 0xa8, 0x2a, 0x0a, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	----------- 
//	 7	----------- 
//	 8	---------- #
//	 9	---------- #
//	10	----------- 
//	11	----------- 
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr176[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x58, 0x55, 0x55, 0x58, 0x55, 0x55, 0x56, 0x55, 0x55, 0x56, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----  -----
//	 2	---  ##  ---
//	 3	-- ##  ## --
//	 4	- #      # -
//	 5	 #        # 
//	 6	#          #
//	 7	#          #
//	 8	     ##     
//	 9	     ##     
//	10	#          #
//	11	#          #
//	12	 #        # 
//	13	- #      # -
//	14	-- ##  ## --
//	15	---  ##  ---
//	16	-----  -----
//	17	------------
//		000000000011
//		012345678901
const char chr177[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x69, 0x55, 0x56, 0x82, 0x95, 0x58, 0x28, 0x25, 0x62, 0xaa, 0x89, 0x8a, 0xaa, 0xa2, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0xaa, 0x82, 0xaa, 0xaa, 0x82, 0xaa, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x8a, 0xaa, 0xa2, 0x62, 0xaa, 0x89, 0x58, 0x28, 0x25, 0x56, 0x82, 0x95, 0x55, 0x69, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	 -----------
//	 7	 -----------
//	 8	# ----------
//	 9	# ----------
//	10	 -----------
//	11	 -----------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr178[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x55, 0x25, 0x55, 0x55, 0x25, 0x55, 0x55, 0x95, 0x55, 0x55, 0x95, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	----    ----
//	 5	---- ## ----
//	 6	---- ## ----
//	 7	---- ## ----
//	 8	     ##     
//	 9	 ########## 
//	10	 #        # 
//	11	 # ##  ## # 
//	12	 # ##  ## # 
//	13	 #        # 
//	14	 #        # 
//	15	 ########## 
//	16	            
//	17	------------
//		000000000011
//		012345678901
const char chr179[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0xaa, 0x55, 0x55, 0x82, 0x55, 0x55, 0x82, 0x55, 0x55, 0x82, 0x55, 0xaa, 0x82, 0xaa, 0x80, 0x00, 0x02, 0x8a, 0xaa, 0xa2, 0x88, 0x28, 0x22, 0x88, 0x28, 0x22, 0x8a, 0xaa, 0xa2, 0x8a, 0xaa, 0xa2, 0x80, 0x00, 0x02, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55};


//	 0	---------- -
//	 1	--------- --
//	 2	--------#---
//	 3	-----##-#---
//	 4	----#####---
//	 5	---######---
//	 6	--########--
//	 7	-##########-
//	 8	##        ##
//	 9	-#    ### #-
//	10	-#    #-# #-
//	11	-#    ### #-
//	12	-#        #-
//	13	-#        #-
//	14	-# ###    #-
//	15	-# #-#    #-
//	16	-# #-#    #-
//	17	-##########-
//		000000000011
//		012345678901
const char chr180[] PROGMEM = {0x55, 0x55, 0x59, 0x55, 0x55, 0x65, 0x55, 0x55, 0x15, 0x55, 0x41, 0x15, 0x55, 0x00, 0x15, 0x54, 0x00, 0x15, 0x50, 0x00, 0x05, 0x40, 0x00, 0x01, 0x0a, 0xaa, 0xa0, 0x4a, 0xa0, 0x21, 0x4a, 0xa1, 0x21, 0x4a, 0xa0, 0x21, 0x4a, 0xaa, 0xa1, 0x4a, 0xaa, 0xa1, 0x48, 0x0a, 0xa1, 0x48, 0x4a, 0xa1, 0x48, 0x4a, 0xa1, 0x40, 0x00, 0x01};


//	 0	#######-----
//	 1	#     #-----
//	 2	# # # #-----
//	 3	# # # #-----
//	 4	#######-----
//	 5	-----# #----
//	 6	----# # #---
//	 7	---# ### #--
//	 8	---#     #--
//	 9	---# ### #--
//	10	---# #-# #--
//	11	----##-###--
//	12	-------# #--
//	13	-------# ###
//	14	-------#   #
//	15	-------# # #
//	16	-------# # #
//	17	-------#####
//		000000000011
//		012345678901
const char chr181[] PROGMEM = {0x00, 0x01, 0x55, 0x2a, 0xa1, 0x55, 0x22, 0x21, 0x55, 0x22, 0x21, 0x55, 0x00, 0x01, 0x55, 0x55, 0x48, 0x55, 0x55, 0x22, 0x15, 0x54, 0x80, 0x85, 0x54, 0xaa, 0x85, 0x54, 0x80, 0x85, 0x54, 0x84, 0x85, 0x55, 0x04, 0x05, 0x55, 0x54, 0x85, 0x55, 0x54, 0x80, 0x55, 0x54, 0xa8, 0x55, 0x54, 0x88, 0x55, 0x54, 0x88, 0x55, 0x54, 0x00};


//	 0	--   -------
//	 1	- ### ---  -
//	 2	- # ## - # -
//	 3	- #  ## # --
//	 4	- ##  ## ---
//	 5	-- ##  ## --
//	 6	--- ##  ## -
//	 7	---- ##  ## 
//	 8	---- ###  # 
//	 9	---- # #### 
//	10	---- # #   -
//	11	---- # # ---
//	12	---- # # ---
//	13	--- ## ## --
//	14	-- #     # -
//	15	- #       # 
//	16	- ######### 
//	17	--         -
//		000000000011
//		012345678901
const char chr182[] PROGMEM = {0x5a, 0x95, 0x55, 0x60, 0x25, 0x69, 0x62, 0x09, 0x89, 0x62, 0x82, 0x25, 0x60, 0xa0, 0x95, 0x58, 0x28, 0x25, 0x56, 0x0a, 0x09, 0x55, 0x82, 0x82, 0x55, 0x80, 0xa2, 0x55, 0x88, 0x02, 0x55, 0x88, 0xa9, 0x55, 0x88, 0x95, 0x55, 0x88, 0x95, 0x56, 0x08, 0x25, 0x58, 0xaa, 0x89, 0x62, 0xaa, 0xa2, 0x60, 0x00, 0x02, 0x5a, 0xaa, 0xa9};


//	 0	-----##-----
//	 1	----#  #----
//	 2	---#    #---
//	 3	--#      #--
//	 4	--###  ###--
//	 5	----#  #----
//	 6	----#  #----
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr183[] PROGMEM = {0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x50, 0x28, 0x05, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-----##-----
//	 2	----#  #----
//	 3	---#    #---
//	 4	--#      #--
//	 5	--###  ###--
//	 6	----#  #----
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr184[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x50, 0x28, 0x05, 0x55, 0x28, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----##-----
//	 4	----#  #----
//	 5	---#    #---
//	 6	--#      #--
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr185[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	-----##-----
//	 5	----#  #----
//	 6	---#    #---
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr186[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr187[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	---#    #---
//	12	----#  #----
//	13	-----##-----
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr188[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	--#      #--
//	12	---#    #---
//	13	----#  #----
//	14	-----##-----
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr189[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	----#  #----
//	12	--###  ###--
//	13	--#      #--
//	14	---#    #---
//	15	----#  #----
//	16	-----##-----
//	17	------------
//		000000000011
//		012345678901
const char chr190[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x28, 0x55, 0x50, 0x28, 0x05, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	----#  #----
//	12	----#  #----
//	13	--###  ###--
//	14	--#      #--
//	15	---#    #---
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr191[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x50, 0x28, 0x05, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	            
//	 1	            
//	 2	            
//	 3	            
//	 4	            
//	 5	            
//	 6	            
//	 7	            
//	 8	            
//	 9	            
//	10	            
//	11	            
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr192[] PROGMEM = {0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	            
//	 1	            
//	 2	            
//	 3	            
//	 4	            
//	 5	            
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr193[] PROGMEM = {0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	            
//	11	            
//	12	            
//	13	            
//	14	            
//	15	            
//	16	            
//	17	            
//		000000000011
//		012345678901
const char chr194[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa};


//	 0	            
//	 1	            
//	 2	            
//	 3	            
//	 4	            
//	 5	            
//	 6	            
//	 7	            
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr195[] PROGMEM = {0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	            
//	13	            
//	14	            
//	15	            
//	16	            
//	17	            
//		000000000011
//		012345678901
const char chr196[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	            
//	 7	            
//	 8	            
//	 9	            
//	10	            
//	11	            
//	12	            
//	13	            
//	14	            
//	15	            
//	16	            
//	17	            
//		000000000011
//		012345678901
const char chr197[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa};


//	 0	------------
//	 1	-##########-
//	 2	-#        #-
//	 3	-# ##  ## #-
//	 4	-# # # #  #-
//	 5	-# ##  #  #-
//	 6	-# #   #  #-
//	 7	-# #   ## #-
//	 8	-#        #-
//	 9	#####  #####
//	10	#          #
//	11	# ######## #
//	12	# # # # #  #
//	13	#  # # # # #
//	14	# # # # #  #
//	15	#  ######  #
//	16	#          #
//	17	############
//		000000000011
//		012345678901
const char chr198[] PROGMEM = {0x55, 0x55, 0x55, 0x40, 0x00, 0x01, 0x4a, 0xaa, 0xa1, 0x48, 0x28, 0x21, 0x48, 0x88, 0xa1, 0x48, 0x28, 0xa1, 0x48, 0xa8, 0xa1, 0x48, 0xa8, 0x21, 0x4a, 0xaa, 0xa1, 0x00, 0x28, 0x00, 0x2a, 0xaa, 0xa8, 0x20, 0x00, 0x08, 0x22, 0x22, 0x28, 0x28, 0x88, 0x88, 0x22, 0x22, 0x28, 0x28, 0x00, 0x28, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	--         -
//	 5	-- ------- -
//	 6	-- ------- -
//	 7	-- ------- -
//	 8	-- ------- -
//	 9	-- ------- -
//	10	--         -
//	11	-- ------- -
//	12	-- ------- -
//	13	-- ------- -
//	14	-- ------- -
//	15	-- ------- -
//	16	--         -
//	17	------------
//		000000000011
//		012345678901
const char chr199[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x5a, 0xaa, 0xa9, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x5a, 0xaa, 0xa9, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x59, 0x55, 0x59, 0x5a, 0xaa, 0xa9, 0x55, 0x55, 0x55};


//	 0	------##----
//	 1	-----#  #---
//	 2	----#    #--
//	 3	----#     #-
//	 4	-----#     #
//	 5	------#    #
//	 6	-------#    
//	 7	--------## -
//	 8	----------# 
//	 9	--------#-  
//	10	------#- -- 
//	11	----#- # #--
//	12	---# # ##  #
//	13	---# ## ##--
//	14	----# ##  #-
//	15	-----# ##---
//	16	------#  #--
//	17	-------##---
//		000000000011
//		012345678901
const char chr200[] PROGMEM = {0x55, 0x50, 0x55, 0x55, 0x4a, 0x15, 0x55, 0x2a, 0x85, 0x55, 0x2a, 0xa1, 0x55, 0x4a, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x54, 0xaa, 0x55, 0x55, 0x09, 0x55, 0x55, 0x52, 0x55, 0x55, 0x1a, 0x55, 0x51, 0x96, 0x55, 0x18, 0x85, 0x54, 0x88, 0x28, 0x54, 0x82, 0x05, 0x55, 0x20, 0xa1, 0x55, 0x48, 0x15, 0x55, 0x52, 0x85, 0x55, 0x54, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	-#----------
//	 6	# #---------
//	 7	   #--------
//	 8	  #---------
//	 9	 - ##-------
//	10	#    #------
//	11	-#    #-----
//	12	-#     #----
//	13	--#     #---
//	14	---#    #---
//	15	----#  #----
//	16	-----##-----
//	17	------------
//		000000000011
//		012345678901
const char chr201[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x45, 0x55, 0x55, 0x21, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa1, 0x55, 0x55, 0x98, 0x15, 0x55, 0x2a, 0x85, 0x55, 0x4a, 0xa1, 0x55, 0x4a, 0xa8, 0x55, 0x52, 0xaa, 0x15, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55};


//	 0	---#     #--
//	 1	--# ##### #-
//	 2	-# ##   ## #
//	 3	- ## ### ## 
//	 4	- # ## ## # 
//	 5	- # #   # # 
//	 6	- # ## ## # 
//	 7	- ## # # ## 
//	 8	-# ### ### #
//	 9	--# ## ## #-
//	10	---#-# #-#--
//	11	-----# #----
//	12	-----# #----
//	13	-----# #----
//	14	-----# #----
//	15	-----# #----
//	16	----#   #---
//	17	---#     #--
//		000000000011
//		012345678901
const char chr202[] PROGMEM = {0x54, 0xaa, 0x85, 0x52, 0x00, 0x21, 0x48, 0x2a, 0x08, 0x60, 0x80, 0x82, 0x62, 0x08, 0x22, 0x62, 0x2a, 0x22, 0x62, 0x08, 0x22, 0x60, 0x88, 0x82, 0x48, 0x08, 0x08, 0x52, 0x08, 0x21, 0x54, 0x48, 0x45, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x2a, 0x15, 0x54, 0xaa, 0x85};


//	 0	------------
//	 1	###---------
//	 2	# #---------
//	 3	# ###-------
//	 4	# # #-------
//	 5	#  ##-------
//	 6	# # #-------
//	 7	#########---
//	 8	--#     #---
//	 9	--# # # #---
//	10	--# # # #---
//	11	--########--
//	12	-------# #--
//	13	-------# ###
//	14	-------#   #
//	15	-------# # #
//	16	-------# # #
//	17	-------#####
//		000000000011
//		012345678901
const char chr203[] PROGMEM = {0x55, 0x55, 0x55, 0x01, 0x55, 0x55, 0x21, 0x55, 0x55, 0x20, 0x15, 0x55, 0x22, 0x15, 0x55, 0x28, 0x15, 0x55, 0x22, 0x15, 0x55, 0x00, 0x00, 0x15, 0x52, 0xaa, 0x15, 0x52, 0x22, 0x15, 0x52, 0x22, 0x15, 0x50, 0x00, 0x05, 0x55, 0x54, 0x85, 0x55, 0x54, 0x80, 0x55, 0x54, 0xa8, 0x55, 0x54, 0x88, 0x55, 0x54, 0x88, 0x55, 0x54, 0x00};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	#########---
//	13	# #  #  #---
//	14	#  #  # #---
//	15	# ## ## #---
//	16	# ## ## #---
//	17	#########---
//		000000000011
//		012345678901
const char chr204[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x15, 0x22, 0x8a, 0x15, 0x28, 0xa2, 0x15, 0x20, 0x82, 0x15, 0x20, 0x82, 0x15, 0x00, 0x00, 0x15};


//	 0	----####----
//	 1	----#  #----
//	 2	----#  #----
//	 3	----#  #----
//	 4	----#  #----
//	 5	----#  #----
//	 6	----#  #----
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	#####  #####
//	12	#          #
//	13	##        ##
//	14	-##      ##-
//	15	--##    ##--
//	16	---##  ##---
//	17	----####----
//		000000000011
//		012345678901
const char chr205[] PROGMEM = {0x55, 0x00, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x00, 0x28, 0x00, 0x2a, 0xaa, 0xa8, 0x0a, 0xaa, 0xa0, 0x42, 0xaa, 0x81, 0x50, 0xaa, 0x05, 0x54, 0x28, 0x15, 0x55, 0x00, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	#          #
//	 9	#          #
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr206[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	----####----
//	 1	---##  ##---
//	 2	--##    ##--
//	 3	-##      ##-
//	 4	##        ##
//	 5	#          #
//	 6	#####  #####
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	----#  #----
//	15	----#  #----
//	16	----#  #----
//	17	----####----
//		000000000011
//		012345678901
const char chr207[] PROGMEM = {0x55, 0x00, 0x55, 0x54, 0x28, 0x15, 0x50, 0xaa, 0x05, 0x42, 0xaa, 0x81, 0x0a, 0xaa, 0xa0, 0x2a, 0xaa, 0xa8, 0x00, 0x28, 0x00, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x00, 0x55};


//	 0	------------
//	 1	---####-----
//	 2	--#    #----
//	 3	-# #### #---
//	 4	-# #--# #---
//	 5	-# #--# #---
//	 6	-# #### #---
//	 7	--#    #----
//	 8	---####-----
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr208[] PROGMEM = {0x55, 0x55, 0x55, 0x54, 0x01, 0x55, 0x52, 0xa8, 0x55, 0x48, 0x02, 0x15, 0x48, 0x52, 0x15, 0x48, 0x52, 0x15, 0x48, 0x02, 0x15, 0x52, 0xa8, 0x55, 0x54, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	---##--#--#-
//	 1	--#  ## ## #
//	 2	-# ## #  # #
//	 3	-# ## # #  #
//	 4	-# ## # #  #
//	 5	--#  ## ## #
//	 6	---##--#--#-
//	 7	-----###----
//	 8	----#   #---
//	 9	---# # # #--
//	10	--# ## ## #-
//	11	-# #-# ### #
//	12	-# #-#   # #
//	13	-# #--#### #
//	14	--# #---# #-
//	15	---# ### #--
//	16	----#   #---
//	17	-----###----
//		000000000011
//		012345678901
const char chr209[] PROGMEM = {0x54, 0x14, 0x51, 0x52, 0x82, 0x08, 0x48, 0x22, 0x88, 0x48, 0x22, 0x28, 0x48, 0x22, 0x28, 0x52, 0x82, 0x08, 0x54, 0x14, 0x51, 0x55, 0x40, 0x55, 0x55, 0x2a, 0x15, 0x54, 0x88, 0x85, 0x52, 0x08, 0x21, 0x48, 0x48, 0x08, 0x48, 0x4a, 0x88, 0x48, 0x50, 0x08, 0x52, 0x15, 0x21, 0x54, 0x80, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x40, 0x55};


//	 0	-###-#--#-#-
//	 1	#   # ## # #
//	 2	# ### ## # #
//	 3	#  ## #-# #-
//	 4	# #-# ### #-
//	 5	# #-#   # #-
//	 6	-#---###-#--
//	 7	-----###----
//	 8	----#   #---
//	 9	---# # # #--
//	10	--# ## ## #-
//	11	-# #-# ### #
//	12	-# #-#   # #
//	13	-# #--#### #
//	14	--# #---# #-
//	15	---# ### #--
//	16	----#   #---
//	17	-----###----
//		000000000011
//		012345678901
const char chr210[] PROGMEM = {0x40, 0x45, 0x11, 0x2a, 0x20, 0x88, 0x20, 0x20, 0x88, 0x28, 0x21, 0x21, 0x21, 0x20, 0x21, 0x21, 0x2a, 0x21, 0x45, 0x40, 0x45, 0x55, 0x40, 0x55, 0x55, 0x2a, 0x15, 0x54, 0x88, 0x85, 0x52, 0x08, 0x21, 0x48, 0x48, 0x08, 0x48, 0x4a, 0x88, 0x48, 0x50, 0x08, 0x52, 0x15, 0x21, 0x54, 0x80, 0x85, 0x55, 0x2a, 0x15, 0x55, 0x40, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--##----##--
//	 4	-#  #--#  #-
//	 5	-#   #-#  #-
//	 6	-#   ###  #-
//	 7	-#    ##  #-
//	 8	##  #  #  ##
//	 9	 #  #  #  # 
//	10	##  ##    ##
//	11	-#  ###   #-
//	12	-#  #-#   #-
//	13	-#  #--#  #-
//	14	--##----##--
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr211[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x55, 0x05, 0x4a, 0x14, 0xa1, 0x4a, 0x84, 0xa1, 0x4a, 0x80, 0xa1, 0x4a, 0xa0, 0xa1, 0x0a, 0x28, 0xa0, 0x8a, 0x28, 0xa2, 0x0a, 0x0a, 0xa0, 0x4a, 0x02, 0xa1, 0x4a, 0x12, 0xa1, 0x4a, 0x14, 0xa1, 0x50, 0x55, 0x05, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	----####----
//	 4	---#    #---
//	 5	--#      #--
//	 6	--#  ### #--
//	 7	--#  #------
//	 8	###     ####
//	 9	   #     #  
//	10	#######  ###
//	11	--# ###  #--
//	12	--#      #--
//	13	--##    ##--
//	14	----#####---
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr212[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x52, 0x80, 0x85, 0x52, 0x85, 0x55, 0x02, 0xaa, 0x00, 0xa8, 0xaa, 0x8a, 0x00, 0x02, 0x80, 0x52, 0x02, 0x85, 0x52, 0xaa, 0x85, 0x50, 0xaa, 0x05, 0x55, 0x00, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	---######---
//	 4	--#      #--
//	 5	--#      #--
//	 6	--#  ####---
//	 7	--#  #------
//	 8	###    #####
//	 9	  #    #    
//	10	###  #######
//	11	--#  ###----
//	12	--#      #--
//	13	--#      #--
//	14	---######---
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr213[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x00, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x52, 0x80, 0x15, 0x52, 0x85, 0x55, 0x02, 0xa8, 0x00, 0xa2, 0xa8, 0xaa, 0x02, 0x80, 0x00, 0x52, 0x80, 0x55, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0x85, 0x54, 0x00, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	--##----##--
//	 4	-#  #--#  #-
//	 5	-#  #--#  #-
//	 6	-#  #--#  #-
//	 7	-#  ####  #-
//	 8	##  #  #  ##
//	 9	  # #  # #  
//	10	### #  # ###
//	11	--#  ##  #--
//	12	--#  ##  #--
//	13	--#  ##  #--
//	14	---######---
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr214[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x55, 0x05, 0x4a, 0x14, 0xa1, 0x4a, 0x14, 0xa1, 0x4a, 0x14, 0xa1, 0x4a, 0x00, 0xa1, 0x0a, 0x28, 0xa0, 0xa2, 0x28, 0x8a, 0x02, 0x28, 0x80, 0x52, 0x82, 0x85, 0x52, 0x82, 0x85, 0x52, 0x82, 0x85, 0x54, 0x00, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	-----#------
//	 7	----# #-----
//	 8	###-# #-####
//	 9	   ## ##    
//	10	###-# #-####
//	11	----# #-----
//	12	-----#------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr215[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x45, 0x55, 0x55, 0x21, 0x55, 0x01, 0x21, 0x00, 0xa8, 0x20, 0xaa, 0x01, 0x21, 0x00, 0x55, 0x21, 0x55, 0x55, 0x45, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	############
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr216[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	--------#---
//	 6	-------# #--
//	 7	-#######  #-
//	 8	#          #
//	 9	#          #
//	10	-#######  #-
//	11	-------# #--
//	12	--------#---
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr217[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x15, 0x55, 0x54, 0x85, 0x40, 0x00, 0xa1, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0xa1, 0x55, 0x54, 0x85, 0x55, 0x55, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----######-
//	 4	----#      #
//	 5	-----#     #
//	 6	------#    #
//	 7	-----#     #
//	 8	----#   #  #
//	 9	---#   #-# #
//	10	--#   #---#-
//	11	-#   #------
//	12	--# #-------
//	13	---#--------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr218[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x01, 0x55, 0x2a, 0xa8, 0x55, 0x4a, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x4a, 0xa8, 0x55, 0x2a, 0x28, 0x54, 0xa8, 0x48, 0x52, 0xa1, 0x51, 0x4a, 0x85, 0x55, 0x52, 0x15, 0x55, 0x54, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----##-----
//	 4	----#  #----
//	 5	---#    #---
//	 6	--#      #--
//	 7	---##  ##---
//	 8	----#  #----
//	 9	----#  #----
//	10	----#  #----
//	11	----#  #----
//	12	----#  #----
//	13	----#  #----
//	14	-----##-----
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr219[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x54, 0x28, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-######-----
//	 4	#      #----
//	 5	#     #-----
//	 6	#    #------
//	 7	#     #-----
//	 8	#  #   #----
//	 9	# #-#   #---
//	10	-#---#   #--
//	11	------#   #-
//	12	-------# #--
//	13	--------#---
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr220[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x01, 0x55, 0x2a, 0xa8, 0x55, 0x2a, 0xa1, 0x55, 0x2a, 0x85, 0x55, 0x2a, 0xa1, 0x55, 0x28, 0xa8, 0x55, 0x21, 0x2a, 0x15, 0x45, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x54, 0x85, 0x55, 0x55, 0x15, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	---#--------
//	 6	--# #-------
//	 7	-#  #######-
//	 8	#          #
//	 9	#          #
//	10	-#  #######-
//	11	--# #-------
//	12	---#--------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr221[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x55, 0x55, 0x52, 0x15, 0x55, 0x4a, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x4a, 0x00, 0x01, 0x52, 0x15, 0x55, 0x54, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	--------#---
//	 5	-------# #--
//	 6	------#   #-
//	 7	-#---#   #--
//	 8	# #-#   #---
//	 9	#  #   #----
//	10	#     #-----
//	11	#    #------
//	12	#     #-----
//	13	#      #----
//	14	-######-----
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr222[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x15, 0x55, 0x54, 0x85, 0x55, 0x52, 0xa1, 0x45, 0x4a, 0x85, 0x21, 0x2a, 0x15, 0x28, 0xa8, 0x55, 0x2a, 0xa1, 0x55, 0x2a, 0x85, 0x55, 0x2a, 0xa1, 0x55, 0x2a, 0xa8, 0x55, 0x40, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----##-----
//	 4	----#  #----
//	 5	----#  #----
//	 6	----#  #----
//	 7	----#  #----
//	 8	----#  #----
//	 9	----#  #----
//	10	---##  ##---
//	11	--#      #--
//	12	---#    #---
//	13	----#  #----
//	14	-----##-----
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr223[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0x28, 0x15, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	---#--------
//	 5	--# #-------
//	 6	-#   #------
//	 7	--#   #---#-
//	 8	---#   #-# #
//	 9	----#   #  #
//	10	-----#     #
//	11	------#    #
//	12	-----#     #
//	13	----#      #
//	14	-----######-
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr224[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x54, 0x55, 0x55, 0x52, 0x15, 0x55, 0x4a, 0x85, 0x55, 0x52, 0xa1, 0x51, 0x54, 0xa8, 0x48, 0x55, 0x2a, 0x28, 0x55, 0x4a, 0xa8, 0x55, 0x52, 0xa8, 0x55, 0x4a, 0xa8, 0x55, 0x2a, 0xa8, 0x55, 0x40, 0x01, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	            
//	 4	############
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr225[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	            
//	 9	            
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr226[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	############
//	14	            
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr227[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	#####--#####
//	14	    #--#    
//	15	     ##     
//	16	####    ####
//	17	---######---
//		000000000011
//		012345678901
const char chr228[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x14, 0x00, 0xaa, 0x14, 0xaa, 0xaa, 0x82, 0xaa, 0x00, 0xaa, 0x00, 0x54, 0x00, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	####----####
//	12	#   #--#   #
//	13	-#   ##   #-
//	14	--#      #--
//	15	---#    #---
//	16	----#  #----
//	17	-----##-----
//		000000000011
//		012345678901
const char chr229[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x55, 0x00, 0x2a, 0x14, 0xa8, 0x4a, 0x82, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55};


//	 0	----##--#-#-
//	 1	---#  ## # #
//	 2	---# # # # #
//	 3	---#  ##   #
//	 4	---# #-# # #
//	 5	-#-# #-# # #
//	 6	# #-#---#-#-
//	 7	   #--------
//	 8	  #---------
//	 9	 - ##-------
//	10	#    #------
//	11	-#    #-----
//	12	-#     #----
//	13	--#     #---
//	14	---#    #---
//	15	----#  #----
//	16	-----##-----
//	17	------------
//		000000000011
//		012345678901
const char chr230[] PROGMEM = {0x55, 0x05, 0x11, 0x54, 0xa0, 0x88, 0x54, 0x88, 0x88, 0x54, 0xa0, 0xa8, 0x54, 0x84, 0x88, 0x44, 0x84, 0x88, 0x21, 0x15, 0x11, 0xa8, 0x55, 0x55, 0xa1, 0x55, 0x55, 0x98, 0x15, 0x55, 0x2a, 0x85, 0x55, 0x4a, 0xa1, 0x55, 0x4a, 0xa8, 0x55, 0x52, 0xaa, 0x15, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55};


//	 0	---------##-
//	 1	--------#  #
//	 2	-------# ##-
//	 3	-------# #--
//	 4	-------# ##-
//	 5	-#------#  #
//	 6	# #-----###-
//	 7	   #---# # #
//	 8	  #----# # #
//	 9	 - ##--#   #
//	10	#    #-# # #
//	11	-#    ## # #
//	12	-#     ##-#-
//	13	--#     #---
//	14	---#    #---
//	15	----#  #----
//	16	-----##-----
//	17	------------
//		000000000011
//		012345678901
const char chr231[] PROGMEM = {0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0x81, 0x55, 0x54, 0x85, 0x55, 0x54, 0x81, 0x45, 0x55, 0x28, 0x21, 0x55, 0x01, 0xa8, 0x54, 0x88, 0xa1, 0x54, 0x88, 0x98, 0x14, 0xa8, 0x2a, 0x84, 0x88, 0x4a, 0xa0, 0x88, 0x4a, 0xa8, 0x11, 0x52, 0xaa, 0x15, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	#          #
//	 9	#          #
//	10	##        ##
//	11	--#      #--
//	12	---#    #---
//	13	----#  #----
//	14	-----##-----
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr232[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x0a, 0xaa, 0xa0, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	############
//	 8	#          #
//	 9	#          #
//	10	#####  #####
//	11	----#  #----
//	12	-####  ####-
//	13	#          #
//	14	-#        #-
//	15	--#      #--
//	16	---#    #---
//	17	----#  #----
//		000000000011
//		012345678901
const char chr233[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x00, 0x28, 0x00, 0x55, 0x28, 0x55, 0x40, 0x28, 0x01, 0x2a, 0xaa, 0xa8, 0x4a, 0xaa, 0xa1, 0x52, 0xaa, 0x85, 0x54, 0xaa, 0x15, 0x55, 0x28, 0x55};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	-----##-----
//	 4	----#  #----
//	 5	---#    #---
//	 6	--#      #--
//	 7	##        ##
//	 8	#          #
//	 9	#          #
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr234[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x0a, 0xaa, 0xa0, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	----#  #----
//	 1	---#    #---
//	 2	--#      #--
//	 3	-#        #-
//	 4	#          #
//	 5	-####  ####-
//	 6	----#  #----
//	 7	#####  #####
//	 8	#          #
//	 9	#          #
//	10	############
//	11	------------
//	12	------------
//	13	------------
//	14	------------
//	15	------------
//	16	------------
//	17	------------
//		000000000011
//		012345678901
const char chr235[] PROGMEM = {0x55, 0x28, 0x55, 0x54, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x4a, 0xaa, 0xa1, 0x2a, 0xaa, 0xa8, 0x40, 0x28, 0x01, 0x55, 0x28, 0x55, 0x00, 0x28, 0x00, 0x2a, 0xaa, 0xa8, 0x2a, 0xaa, 0xa8, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	-###########
//	 2	#           
//	 3	# ######## #
//	 4	# # #--# # #
//	 5	# # #-#  # #
//	 6	# # ##   # #
//	 7	# # #    # #
//	 8	# #      # #
//	 9	# #      # #
//	10	# # #    # #
//	11	# # ##   # #
//	12	# # #-#  # #
//	13	# # #--# # #
//	14	# ######## #
//	15	#           
//	16	-###########
//	17	------------
//		000000000011
//		012345678901
const char chr236[] PROGMEM = {0x55, 0x55, 0x55, 0x40, 0x00, 0x00, 0x2a, 0xaa, 0xaa, 0x20, 0x00, 0x08, 0x22, 0x14, 0x88, 0x22, 0x12, 0x88, 0x22, 0x0a, 0x88, 0x22, 0x2a, 0x88, 0x22, 0xaa, 0x88, 0x22, 0xaa, 0x88, 0x22, 0x2a, 0x88, 0x22, 0x0a, 0x88, 0x22, 0x12, 0x88, 0x22, 0x14, 0x88, 0x20, 0x00, 0x08, 0x2a, 0xaa, 0xaa, 0x40, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	----------##
//	 2	---------#  
//	 3	---------# #
//	 4	---------# #
//	 5	---------# #
//	 6	---------# #
//	 7	---------# #
//	 8	---------# #
//	 9	---------# #
//	10	---------# #
//	11	---------# #
//	12	---------# #
//	13	---------# #
//	14	---------# #
//	15	---------#  
//	16	----------##
//	17	------------
//		000000000011
//		012345678901
const char chr237[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x55, 0x55, 0x4a, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x48, 0x55, 0x55, 0x4a, 0x55, 0x55, 0x50, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	############
//	 4	            
//	 5	            
//	 6	            
//	 7	            
//	 8	            
//	 9	            
//	10	            
//	11	            
//	12	            
//	13	            
//	14	############
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr238[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	############
//	 4	         #--
//	 5	         #--
//	 6	         #--
//	 7	         #--
//	 8	         #--
//	 9	         #--
//	10	         #--
//	11	         #--
//	12	         #--
//	13	         #--
//	14	############
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr239[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0xaa, 0xaa, 0x85, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	############
//	 4	      #-----
//	 5	      #-----
//	 6	      #-----
//	 7	      #-----
//	 8	      #-----
//	 9	      #-----
//	10	      #-----
//	11	      #-----
//	12	      #-----
//	13	      #-----
//	14	############
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr240[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	############
//	 4	   #--------
//	 5	   #--------
//	 6	   #--------
//	 7	   #--------
//	 8	   #--------
//	 9	   #--------
//	10	   #--------
//	11	   #--------
//	12	   #--------
//	13	   #--------
//	14	############
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr241[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0xa8, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	############
//	 2	            
//	 3	############
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	------------
//	 8	------------
//	 9	------------
//	10	------------
//	11	------------
//	12	------------
//	13	------------
//	14	############
//	15	            
//	16	############
//	17	------------
//		000000000011
//		012345678901
const char chr242[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	##----------
//	 2	  #---------
//	 3	# #---------
//	 4	# #---------
//	 5	# #---------
//	 6	# #---------
//	 7	# #---------
//	 8	# #---------
//	 9	# #---------
//	10	# #---------
//	11	# #---------
//	12	# #---------
//	13	# #---------
//	14	# #---------
//	15	  #---------
//	16	##----------
//	17	------------
//		000000000011
//		012345678901
const char chr243[] PROGMEM = {0x55, 0x55, 0x55, 0x05, 0x55, 0x55, 0xa1, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0x21, 0x55, 0x55, 0xa1, 0x55, 0x55, 0x05, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	###########-
//	 2	           #
//	 3	# ######## #
//	 4	# # #--# # #
//	 5	# #  #-# # #
//	 6	# #   ## # #
//	 7	# #    # # #
//	 8	# #      # #
//	 9	# #      # #
//	10	# #    # # #
//	11	# #   ## # #
//	12	# #  #-# # #
//	13	# # #--# # #
//	14	# ######## #
//	15	           #
//	16	###########-
//	17	------------
//		000000000011
//		012345678901
const char chr244[] PROGMEM = {0x55, 0x55, 0x55, 0x00, 0x00, 0x01, 0xaa, 0xaa, 0xa8, 0x20, 0x00, 0x08, 0x22, 0x14, 0x88, 0x22, 0x84, 0x88, 0x22, 0xa0, 0x88, 0x22, 0xa8, 0x88, 0x22, 0xaa, 0x88, 0x22, 0xaa, 0x88, 0x22, 0xa8, 0x88, 0x22, 0xa0, 0x88, 0x22, 0x84, 0x88, 0x22, 0x14, 0x88, 0x20, 0x00, 0x08, 0xaa, 0xaa, 0xa8, 0x00, 0x00, 0x01, 0x55, 0x55, 0x55};


//	 0	-----------#
//	 1	----------# 
//	 2	------##--# 
//	 3	-----#  #-##
//	 4	-----#  ##  
//	 5	------###   
//	 6	-------#  ##
//	 7	----###  #--
//	 8	---#  #  #--
//	 9	---#  #  #--
//	10	----###  #--
//	11	-------#  ##
//	12	------###   
//	13	-----#  ##  
//	14	-----#  #-##
//	15	------##--# 
//	16	----------# 
//	17	----------##
//		000000000011
//		012345678901
const char chr245[] PROGMEM = {0x55, 0x55, 0x54, 0x55, 0x55, 0x52, 0x55, 0x50, 0x52, 0x55, 0x4a, 0x10, 0x55, 0x4a, 0x0a, 0x55, 0x50, 0x2a, 0x55, 0x54, 0xa0, 0x55, 0x02, 0x85, 0x54, 0xa2, 0x85, 0x54, 0xa2, 0x85, 0x55, 0x02, 0x85, 0x55, 0x54, 0xa0, 0x55, 0x50, 0x2a, 0x55, 0x4a, 0x0a, 0x55, 0x4a, 0x10, 0x55, 0x50, 0x52, 0x55, 0x55, 0x52, 0x55, 0x55, 0x50};


//	 0	#-----------
//	 1	 #----------
//	 2	 #--##------
//	 3	##-#  #-----
//	 4	  ##  #-----
//	 5	   ###------
//	 6	##  #-------
//	 7	--#  ###----
//	 8	--#  #  #---
//	 9	--#  #  #---
//	10	--#  ###----
//	11	##  #-------
//	12	   ###------
//	13	  ##  #-----
//	14	##-#  #-----
//	15	 #--##------
//	16	 #----------
//	17	#-----------
//		000000000011
//		012345678901
const char chr246[] PROGMEM = {0x15, 0x55, 0x55, 0x85, 0x55, 0x55, 0x85, 0x05, 0x55, 0x04, 0xa1, 0x55, 0xa0, 0xa1, 0x55, 0xa8, 0x05, 0x55, 0x0a, 0x15, 0x55, 0x52, 0x80, 0x55, 0x52, 0x8a, 0x15, 0x52, 0x8a, 0x15, 0x52, 0x80, 0x55, 0x0a, 0x15, 0x55, 0xa8, 0x05, 0x55, 0xa0, 0xa1, 0x55, 0x04, 0xa1, 0x55, 0x85, 0x05, 0x55, 0x85, 0x55, 0x55, 0x15, 0x55, 0x55};


//	 0	------------
//	 1	----------##
//	 2	--------##  
//	 3	-------#    
//	 4	------#   ##
//	 5	-----#  ##-#
//	 6	-----#  #--#
//	 7	----#  #---#
//	 8	----#  #---#
//	 9	----#  #---#
//	10	----#  #---#
//	11	-----#  #--#
//	12	-----#  ##-#
//	13	------#   ##
//	14	-------#    
//	15	--------##  
//	16	----------##
//	17	------------
//		000000000011
//		012345678901
const char chr247[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x55, 0x55, 0x0a, 0x55, 0x54, 0xaa, 0x55, 0x52, 0xa0, 0x55, 0x4a, 0x04, 0x55, 0x4a, 0x14, 0x55, 0x28, 0x54, 0x55, 0x28, 0x54, 0x55, 0x28, 0x54, 0x55, 0x28, 0x54, 0x55, 0x4a, 0x14, 0x55, 0x4a, 0x04, 0x55, 0x52, 0xa0, 0x55, 0x54, 0xaa, 0x55, 0x55, 0x0a, 0x55, 0x55, 0x50, 0x55, 0x55, 0x55};


//	 0	------------
//	 1	##----------
//	 2	  ##--------
//	 3	    #-------
//	 4	     #------
//	 5	      #-----
//	 6	      #-----
//	 7	       #----
//	 8	       #----
//	 9	       #----
//	10	       #----
//	11	      #-----
//	12	      #-----
//	13	     #------
//	14	    #-------
//	15	  ##--------
//	16	##----------
//	17	------------
//		000000000011
//		012345678901
const char chr248[] PROGMEM = {0x55, 0x55, 0x55, 0x05, 0x55, 0x55, 0xa0, 0x55, 0x55, 0xaa, 0x15, 0x55, 0xaa, 0x85, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa8, 0x55, 0xaa, 0xa8, 0x55, 0xaa, 0xa8, 0x55, 0xaa, 0xa8, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0xa1, 0x55, 0xaa, 0x85, 0x55, 0xaa, 0x15, 0x55, 0xa0, 0x55, 0x55, 0x05, 0x55, 0x55, 0x55, 0x55, 0x55};


//	 0	---------##-
//	 1	--------#  #
//	 2	-------#   #
//	 3	------#  # #
//	 4	-#####  ## #
//	 5	#      #-# #
//	 6	# ##  #--# #
//	 7	# ##  #--# #
//	 8	# ##  #--# #
//	 9	# ##  #--# #
//	10	# ##  #--# #
//	11	#      #-# #
//	12	-#####  ## #
//	13	------#  # #
//	14	-------#   #
//	15	--------#  #
//	16	---------##-
//	17	------------
//		000000000011
//		012345678901
const char chr249[] PROGMEM = {0x55, 0x55, 0x41, 0x55, 0x55, 0x28, 0x55, 0x54, 0xa8, 0x55, 0x52, 0x88, 0x40, 0x0a, 0x08, 0x2a, 0xa8, 0x48, 0x20, 0xa1, 0x48, 0x20, 0xa1, 0x48, 0x20, 0xa1, 0x48, 0x20, 0xa1, 0x48, 0x20, 0xa1, 0x48, 0x2a, 0xa8, 0x48, 0x40, 0x0a, 0x08, 0x55, 0x52, 0x88, 0x55, 0x54, 0xa8, 0x55, 0x55, 0x28, 0x55, 0x55, 0x41, 0x55, 0x55, 0x55};


//	 0	------#-----
//	 1	-----  #----
//	 2	------  #---
//	 3	---##-# #---
//	 4	--#  ## #---
//	 5	-#-# ##  #--
//	 6	# ##  ## #--
//	 7	#  ## ## #--
//	 8	-# ## ## #--
//	 9	#  ## ## #--
//	10	# ##  ## #--
//	11	-#-# ##  #--
//	12	--#  ## #---
//	13	---##-# #---
//	14	------  #---
//	15	-----  #----
//	16	------#-----
//	17	------------
//		000000000011
//		012345678901
const char chr250[] PROGMEM = {0x55, 0x51, 0x55, 0x55, 0x68, 0x55, 0x55, 0x5a, 0x15, 0x54, 0x12, 0x15, 0x52, 0x82, 0x15, 0x44, 0x82, 0x85, 0x20, 0xa0, 0x85, 0x28, 0x20, 0x85, 0x48, 0x20, 0x85, 0x28, 0x20, 0x85, 0x20, 0xa0, 0x85, 0x44, 0x82, 0x85, 0x52, 0x82, 0x15, 0x54, 0x12, 0x15, 0x55, 0x5a, 0x15, 0x55, 0x68, 0x55, 0x55, 0x51, 0x55, 0x55, 0x55, 0x55};


//	 0	-##########-
//	 1	#          #
//	 2	-# ###### #-
//	 3	-# #----# #-
//	 4	-# #----# #-
//	 5	-# #----# #-
//	 6	--# #--# #--
//	 7	---# ## #---
//	 8	----#  #----
//	 9	----#  #----
//	10	---# ## #---
//	11	--# #--# #--
//	12	-# #----# #-
//	13	-# #----# #-
//	14	-# #----# #-
//	15	-# ###### #-
//	16	#          #
//	17	-##########-
//		000000000011
//		012345678901
const char chr251[] PROGMEM = {0x40, 0x00, 0x01, 0x2a, 0xaa, 0xa8, 0x48, 0x00, 0x21, 0x48, 0x55, 0x21, 0x48, 0x55, 0x21, 0x48, 0x55, 0x21, 0x52, 0x14, 0x85, 0x54, 0x82, 0x15, 0x55, 0x28, 0x55, 0x55, 0x28, 0x55, 0x54, 0x82, 0x15, 0x52, 0x14, 0x85, 0x48, 0x55, 0x21, 0x48, 0x55, 0x21, 0x48, 0x55, 0x21, 0x48, 0x00, 0x21, 0x2a, 0xaa, 0xa8, 0x40, 0x00, 0x01};


//	 0	------------
//	 1	--##--------
//	 2	--# #-------
//	 3	--#  #------
//	 4	--#   #-----
//	 5	--#    #----
//	 6	--#     #---
//	 7	--#      #--
//	 8	--#       #-
//	 9	--#        #
//	10	--#     ####
//	11	--#     ##--
//	12	--# ##   #--
//	13	--####   ##-
//	14	------#   #-
//	15	------#   #-
//	16	------## ##-
//	17	--------#---
//		000000000011
//		012345678901
const char chr252[] PROGMEM = {0x55, 0x55, 0x55, 0x50, 0x55, 0x55, 0x52, 0x15, 0x55, 0x52, 0x85, 0x55, 0x52, 0xa1, 0x55, 0x52, 0xa8, 0x55, 0x52, 0xaa, 0x15, 0x52, 0xaa, 0x85, 0x52, 0xaa, 0xa1, 0x52, 0xaa, 0xa8, 0x52, 0xaa, 0x00, 0x52, 0xaa, 0x05, 0x52, 0x0a, 0x85, 0x50, 0x0a, 0x81, 0x55, 0x52, 0xa1, 0x55, 0x52, 0xa1, 0x55, 0x50, 0x81, 0x55, 0x55, 0x15};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	--##--#---#-
//	 8	-#  ## #-# #
//	 9	# ## #  #  #
//	10	# ## # # # #
//	11	# ## # # # #
//	12	#    # ### #
//	13	# ## # #-# #
//	14	# ## # #-# #
//	15	# ## # #-# #
//	16	# ## # #-# #
//	17	-#--#-#---#-
//		000000000011
//		012345678901
const char chr253[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x50, 0x51, 0x51, 0x4a, 0x08, 0x48, 0x20, 0x8a, 0x28, 0x20, 0x88, 0x88, 0x20, 0x88, 0x88, 0x2a, 0x88, 0x08, 0x20, 0x88, 0x48, 0x20, 0x88, 0x48, 0x20, 0x88, 0x48, 0x20, 0x88, 0x48, 0x45, 0x11, 0x51};


//	 0	------------
//	 1	------------
//	 2	------------
//	 3	------------
//	 4	------------
//	 5	------------
//	 6	------------
//	 7	-###--#---#-
//	 8	#   ## #-# #
//	 9	# ## #  #  #
//	10	# ## # # # #
//	11	# ## # # # #
//	12	#   ## ### #
//	13	# ##-# #-# #
//	14	# #--# #-# #
//	15	# #--# #-# #
//	16	# #--# #-# #
//	17	-#----#---#-
//		000000000011
//		012345678901
const char chr254[] PROGMEM = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x40, 0x51, 0x51, 0x2a, 0x08, 0x48, 0x20, 0x8a, 0x28, 0x20, 0x88, 0x88, 0x20, 0x88, 0x88, 0x2a, 0x08, 0x08, 0x20, 0x48, 0x48, 0x21, 0x48, 0x48, 0x21, 0x48, 0x48, 0x21, 0x48, 0x48, 0x45, 0x51, 0x51};


//	 0	            
//	 1	            
//	 2	            
//	 3	            
//	 4	            
//	 5	            
//	 6	            
//	 7	            
//	 8	            
//	 9	            
//	10	            
//	11	            
//	12	            
//	13	            
//	14	            
//	15	            
//	16	            
//	17	            
//		000000000011
//		012345678901
const char chr255[] PROGMEM = {0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa};


const char *chars[] PROGMEM = {chr0, chr1, chr2, chr3, chr4, chr5, chr6, chr7, chr8, chr9, chr10, chr11, chr12, chr13, chr14, chr15, chr16, chr17, chr18, chr19, chr20, chr21, chr22, chr23, chr24, chr25, chr26, chr27, chr28, chr29, chr30, chr31, chr32, chr33, chr34, chr35, chr36, chr37, chr38, chr39, chr40, chr41, chr42, chr43, chr44, chr45, chr46, chr47, chr48, chr49, chr50, chr51, chr52, chr53, chr54, chr55, chr56, chr57, chr58, chr59, chr60, chr61, chr62, chr63, chr64, chr65, chr66, chr67, chr68, chr69, chr70, chr71, chr72, chr73, chr74, chr75, chr76, chr77, chr78, chr79, chr80, chr81, chr82, chr83, chr84, chr85, chr86, chr87, chr88, chr89, chr90, chr91, chr92, chr93, chr94, chr95, chr96, chr97, chr98, chr99, chr100, chr101, chr102, chr103, chr104, chr105, chr106, chr107, chr108, chr109, chr110, chr111, chr112, chr113, chr114, chr115, chr116, chr117, chr118, chr119, chr120, chr121, chr122, chr123, chr124, chr125, chr126, chr127, chr128, chr129, chr130, chr131, chr132, chr133, chr134, chr135, chr136, chr137, chr138, chr139, chr140, chr141, chr142, chr143, chr144, chr145, chr146, chr147, chr148, chr149, chr150, chr151, chr152, chr153, chr154, chr155, chr156, chr157, chr158, chr159, chr160, chr161, chr162, chr163, chr164, chr165, chr166, chr167, chr168, chr169, chr170, chr171, chr172, chr173, chr174, chr175, chr176, chr177, chr178, chr179, chr180, chr181, chr182, chr183, chr184, chr185, chr186, chr187, chr188, chr189, chr190, chr191, chr192, chr193, chr194, chr195, chr196, chr197, chr198, chr199, chr200, chr201, chr202, chr203, chr204, chr205, chr206, chr207, chr208, chr209, chr210, chr211, chr212, chr213, chr214, chr215, chr216, chr217, chr218, chr219, chr220, chr221, chr222, chr223, chr224, chr225, chr226, chr227, chr228, chr229, chr230, chr231, chr232, chr233, chr234, chr235, chr236, chr237, chr238, chr239, chr240, chr241, chr242, chr243, chr244, chr245, chr246, chr247, chr248, chr249, chr250, chr251, chr252, chr253, chr254, chr255};

/**
 * put the above chars in the memory of the MAX7456
 * TODO: read out character, compare and write only if different
 */
void learn_all_chars_pgm(void) {
    char *pointer;
    for (uint16_t c = 0; c < 256; c++) {
        LED3_ON
        // select character to write (CMAH)
        spi_send_byte(0x09, c);

        _delay_ms(1);

        pointer = (char*)pgm_read_word(&chars[c]);

        for (uint8_t i = 0; i < 54; i++) {
            // select 4pixel byte of char (CMAL)
            spi_send_byte(0x0A, i);

            _delay_ms(1);

            // write 4pixel byte of char (CMDI)
            spi_send_byte(0x0B, pgm_read_byte(pointer++));
            _delay_ms(1);
        }

        // write to the NVM array from the shadow RAM (CMM)
        spi_send_byte(0x08, 0b10100000);

        LED3_OFF
        // according to maxim writing to nvram takes about 12ms, lets wait longer
        _delay_ms(50);
    }
}



#endif
